from ..app import App
from ..models.entity_response import EntityResponse
from .filters import IntakeDateFilter, IntakeFilter
from .requests import IntakeRequest
from .responses import IntakeDateResponse, IntakeEntityResponse


class IntakeApp(App):
    """Класс для работы с заявками на вызов курьера"""

    def get_call_dates(self, date_request: IntakeDateFilter) -> IntakeDateResponse:
        """Получить доступные даты вызова курьера

        Args:
            date_request (IntakeDateFilter): фильтр для получения
                доступных дат вызова курьера

        Returns:
            IntakeDateResponse: объект с информацией о доступных датах вызова курьера

        Raises:
            ValueError: если date_request не является объектом IntakeDateFilter
        """
        if not isinstance(date_request, IntakeDateFilter):
            raise ValueError("date_request must be a IntakeDateRequest")
        response = self._get("intakes/availableDays", params=date_request)
        return IntakeDateResponse.model_validate(response)

    def update(self, intake: IntakeFilter) -> EntityResponse:
        """Изменить параметры существующей заявки

        Args:
            intake (IntakeFilter): фильтр для изменения параметров существующей заявки

        Returns:
            EntityResponse: объект с информацией о измененной заявке

        Raises:
            ValueError: если intake не является объектом IntakeFilter
        """
        if not isinstance(intake, IntakeFilter):
            raise ValueError("intake must be a IntakeFilter")
        response = self._patch("intakes", json=intake)
        return EntityResponse.model_validate(response)

    def create(self, intake: IntakeRequest) -> EntityResponse:
        """Создание заявки на вызов курьера

        Args:
            intake (IntakeRequest): объект с информацией о заявке на вызов курьера

        Returns:
            EntityResponse: объект с информацией о созданной заявке

        Raises:
            ValueError: если intake не является объектом IntakeRequest
        """
        if not isinstance(intake, IntakeRequest):
            raise ValueError("intake must be a IntakeRequest")
        response = self._post("intakes", json=intake)
        return EntityResponse.model_validate(response)

    def get(self, uuid: str) -> IntakeEntityResponse:
        """Информация о заявке на вызов курьера

        Args:
            uuid (str): идентификатор заявки на вызов курьера

        Returns:
            IntakeEntityResponse: объект с информацией о заявке на вызов курьера
        """
        response = self._get(f"intakes/{uuid}")
        return IntakeEntityResponse.model_validate(response)

    def delete(self, uuid: str) -> EntityResponse:
        """Удаление заявки на вызов курьера

        Args:
            uuid (str): идентификатор заявки на вызов курьера

        Returns:
            EntityResponse: объект с информацией о удаленной заявке
        """
        response = self._delete(f"intakes/{uuid}")
        return EntityResponse.model_validate(response)



from __future__ import annotations

import time
from collections.abc import Callable
from typing import TYPE_CHECKING, Any, Literal

import requests  # type: ignore

from cdek import constants
from cdek.exceptions import CdekAuthException, CdekRequestException

if TYPE_CHECKING:
    from cdek.client import CdekClient


class App:
    """Базовый класс для всех приложений API CDEK"""

    def __init__(self, client: CdekClient):
        """Сохранить клиент и подготовить HTTP-сессию"""
        self.client = client
        self.constants = constants
        self.token = None
        self.expire = 0
        self._response_headers: dict[str, str] = {}
        self._session = requests.Session()

    def _prepare_data(self, params: Any) -> dict[str, Any] | None:
        """Подготовка параметра для запроса"""
        if params is None:
            return None
        return (
            params.model_dump(exclude_none=True)
            if hasattr(params, "model_dump")
            else params
        )

    def _prepare_header(self, url: str) -> dict[str, str]:
        """Сформировать заголовки запроса с учётом типа ответа"""
        headers = {"Authorization": f"Bearer {self.token}"}

        if ".pdf" in url:
            headers["Accept"] = "application/pdf"
        else:
            headers["Accept"] = "application/json"

        return headers

    def _request(
        self,
        method: Literal["GET", "POST", "PATCH", "DELETE"],
        url: str,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Any:
        """
        Выполнить HTTP-запрос к API CDEK

        Args:
            method: HTTP-метод запроса
            url: относительный путь до ресурса API
            params: параметры строки запроса
            json: тело запроса в формате JSON
            **kwargs: дополнительные параметры `requests.Session.request`

        Returns:
            dict | bytes: JSON-ответ API либо содержимое файла

        Raises:
            CdekRequestException: при сетевых ошибках или ошибках API
        """
        headers = self._prepare_header(url)
        response_json = None
        try:
            response = self._session.request(
                method,
                url=f"{self.client.base_url}{url}",
                params=params,
                json=json,
                headers=headers,
                timeout=self.client.timeout,
                **kwargs,
            )
            if ".pdf" in url and response.status_code == 200:
                return response.content

            response_json = response.json()
            self._check_errors(url, response, response_json)
            self._response_headers = response.headers
            return response_json
        except CdekRequestException:
            raise
        except requests.exceptions.RequestException as e:
            raise CdekRequestException(
                f"Ошибка сети при вызове метода {url}: {str(e)}", response=response_json
            ) from e
        except Exception as e:
            raise CdekRequestException(
                f"Ошибка при вызове метода {url}: {str(e)}"
            ) from e

    def _http_method(
        self,
        method: Literal["GET", "POST", "PATCH", "DELETE"],
        url: str,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Any:
        """
        Преобразовать данные и выполнить запрос с авторизацией

        Args:
            method: используемый HTTP-метод
            url: относительный путь до ресурса API
            params: объект с параметрами запроса
            json: объект, представляющий тело запроса
            **kwargs: дополнительные параметры передачи в `_request`

        Returns:
            dict | bytes: ответ API в зависимости от вызываемого метода
        """
        params = self._prepare_data(params)
        json = self._prepare_data(json)

        # Авторизуемся или получаем данные из кэша
        if not self._check_saved_token():
            self._authorize()

        return self._request(method, url, params, json, **kwargs)

    def _get(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Any:
        """Обёртка над `_http_method` для выполнения GET-запроса"""
        return self._http_method("GET", url, params, json, **kwargs)

    def _post(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Any:
        """Обёртка над `_http_method` для выполнения POST-запроса"""
        return self._http_method("POST", url, params, json, **kwargs)

    def _patch(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Any:
        """Обёртка над `_http_method` для выполнения PATCH-запроса"""
        return self._http_method("PATCH", url, params, json, **kwargs)

    def _delete(
        self,
        url: str,
        params: dict[str, Any] | None = None,
        json: dict[str, Any] | None = None,
        **kwargs: Any,
    ) -> Any:
        """Обёртка над `_http_method` для выполнения DELETE-запроса"""
        return self._http_method("DELETE", url, params, json, **kwargs)

    def _authorize(self) -> bool:
        """
        Авторизация клиента в сервисе Интеграции

        Raises:
            CdekAuthException: в случае ошибки авторизации
        """
        params = {
            "grant_type": "client_credentials",
            "client_id": self.client.account,
            "client_secret": self.client.secure,
        }

        headers = {"Content-Type": "application/x-www-form-urlencoded"}

        try:
            response = self._session.post(
                f"{self.client.base_url}oauth/token",
                data=params,
                headers=headers,
                timeout=self.client.timeout,
            )

            if response.status_code == 200:
                token_info = response.json()
                self.token = token_info.get("access_token", "")
                expires_in = token_info.get("expires_in", 0)
                self.expire = int(time.time()) + expires_in - 10

                if self.client.memory_save_callback is not None:
                    self._save_token(self.client.memory_save_callback)

                return True
            else:
                raise CdekAuthException(constants.AUTH_FAIL)

        except requests.exceptions.RequestException as e:
            raise CdekAuthException(f"{constants.AUTH_FAIL}: {str(e)}") from e

    def _check_saved_token(self) -> bool:
        """Проверить сохранённый токен"""
        check_memory = self.client.memory

        # Если не передан верный сохранённый массив данных для авторизации
        if (
            not check_memory
            or "account_type" not in check_memory
            or "expires_in" not in check_memory
            or "access_token" not in check_memory
        ):
            return False

        # Проверяем тип аккаунта
        if check_memory.get("account_type") != self.client.account_type:
            return False

        # Проверяем срок действия токена
        if check_memory["expires_in"] > time.time() and check_memory.get(
            "access_token"
        ):
            self.token = check_memory["access_token"]
            return True

        return False

    def _save_token(self, callback: Callable[[dict[str, Any]], None]) -> None:
        """Сохранить токен через коллбэк"""
        callback(
            {
                "cdekAuth": {
                    "expires_in": self.expire,
                    "access_token": self.token,
                    "account_type": self.client.account_type,
                }
            }
        )

    def _check_errors(
        self, method: str, response: requests.Response, api_response: dict[str, Any]
    ) -> None:
        """
        Проверить ответ на ошибки

        Args:
            method: URL метода
            response: HTTP ответ
            api_response: распарсенный JSON ответ

        Raises:
            CdekRequestException: в случае ошибки
        """

        # Обработка ошибок с requests
        if response.status_code > 202 and "requests" in api_response:
            if (
                "errors" in api_response["requests"][0]
                or api_response["requests"][0].get("state") == "INVALID"
            ):
                error_data = api_response["requests"][0]["errors"][0]
                message = CdekRequestException.get_translation(
                    error_data["code"], error_data["message"]
                )
                raise CdekRequestException(
                    f"От API CDEK при вызове метода {method} "
                    f"получена ошибка: {message}",
                    response.status_code,
                    api_response,
                )

        # Обработка общих ошибок
        if isinstance(api_response, dict) and (
            ("errors" in api_response and response.status_code == 200)
            or (api_response.get("state") == "INVALID")
            or ("errors" in api_response and response.status_code != 200)
        ):
            error_data = api_response["errors"][0]
            message = CdekRequestException.get_translation(
                error_data["code"], error_data["message"]
            )
            raise CdekRequestException(
                f"От API CDEK при вызове метода {method} получена ошибка: {message}",
                response.status_code,
                api_response,
            )

        # Общая ошибка при неверном статусе
        if response.status_code > 202 and "requests" not in api_response:
            raise CdekRequestException(
                f"Неверный код ответа от сервера CDEK при вызове метода "
                f"{method}: {response.status_code}",
                response.status_code,
                api_response,
            )



from __future__ import annotations

from collections.abc import Callable
from typing import Any

from .apps.agreement import AgreementApp
from .apps.check import CheckApp
from .apps.currency import CurrencyApp
from .apps.form import BarcodeApp, InvoiceApp
from .apps.intake import IntakeApp
from .apps.location import LocationApp
from .apps.office import OfficeApp
from .apps.order import OrderApp
from .apps.payment import PaymentApp
from .apps.tariff import TariffApp
from .apps.webhook import WebhookApp


class CdekClient:
    """Клиент взаимодействия с API CDEK 2.0"""

    def __init__(self, account: str, secure: str | None = None, timeout: float = 5.0):
        """
        Конструктор клиента

        Args:
            account: Логин Account в сервисе Интеграции
            secure: Пароль Secure password в сервисе Интеграции
            timeout: Настройка клиента задающая общий тайм-аут запроса в секундах
        """
        if account == "TEST":
            self.base_url: str = "https://api.edu.cdek.ru/v2/"
            self.account: str = "wqGwiQx0gg8mLtiEKsUinjVSICCjtTEP"
            self.secure: str | None = "RmAmgvSgSl1yirlz9QupbzOJVqhCxcP5"
            self.account_type: str = "TEST"
        else:
            self.base_url = "https://api.cdek.ru/v2/"
            self.account = account
            self.secure = secure
            self.account_type = "COMBAT"

        self.timeout: float = timeout
        self.token: str | None = None
        self.expire: int = 0
        self.memory: dict[str, Any] | None = None
        self.memory_save_callback: Callable[[dict[str, Any]], None] | None = None

    def set_memory(
        self,
        memory: dict[str, Any] | None,
        save_callback: Callable[[dict[str, Any]], None],
    ) -> CdekClient:
        """
        Установить параметры сохранения токена

        Args:
            memory: массив настройки сохранения
            save_callback: коллбэк сохранения

        Returns:
            self
        """
        self.memory = memory
        self.memory_save_callback = save_callback
        return self

    @property
    def currency(self) -> CurrencyApp:
        """Получение информации о валюте"""
        return CurrencyApp(self)

    @property
    def office(self) -> OfficeApp:
        """Получение информации о ПВЗ"""
        return OfficeApp(self)

    @property
    def order(self) -> OrderApp:
        """Получение информации о заказе"""
        return OrderApp(self)

    @property
    def location(self) -> LocationApp:
        """Получение информации о местоположении"""
        return LocationApp(self)

    @property
    def tariff(self) -> TariffApp:
        """Получение информации о тарифе"""
        return TariffApp(self)

    @property
    def barcode(self) -> BarcodeApp:
        """Получение информации о штрих-коде"""
        return BarcodeApp(self)

    @property
    def invoice(self) -> InvoiceApp:
        """Получение информации о накладной"""
        return InvoiceApp(self)

    @property
    def payment(self) -> PaymentApp:
        """Получение информации о платеже"""
        return PaymentApp(self)

    @property
    def agreement(self) -> AgreementApp:
        """Получение информации о договоренности"""
        return AgreementApp(self)

    @property
    def intake(self) -> IntakeApp:
        """Получение информации о заявке на вызов курьера"""
        return IntakeApp(self)

    @property
    def check(self) -> CheckApp:
        """Получение информации о чеке"""
        return CheckApp(self)

    @property
    def webhook(self) -> WebhookApp:
        """Получение информации о вебхуке"""
        return WebhookApp(self)



from .order import OrderApp
from .requests import OrderRequest, OrderUpdateRequest
from .responses import OrderEntityResponse, OrderResponse

__all__ = [
    "OrderApp",
    "OrderRequest",
    "OrderResponse",
    "OrderUpdateRequest",
    "OrderEntityResponse",
]



from __future__ import annotations

import pytest

import cdek
from cdek.apps.agreement import AgreementApp
from cdek.apps.check import CheckApp
from cdek.apps.currency import CurrencyApp
from cdek.apps.form import BarcodeApp, InvoiceApp
from cdek.apps.intake import IntakeApp
from cdek.apps.location import LocationApp
from cdek.apps.office import OfficeApp
from cdek.apps.order import OrderApp
from cdek.apps.payment import PaymentApp
from cdek.apps.tariff import TariffApp
from cdek.apps.webhook import WebhookApp
from cdek.client import CdekClient


def test_public_api_exports_expected_names() -> None:
    expected_symbols = {
        "CdekClient",
        "CdekException",
        "CdekAuthException",
        "CdekRequestException",
        "constants",
    }

    for symbol in expected_symbols:
        assert symbol in cdek.__all__
        assert hasattr(cdek, symbol)


def test_client_initializes_test_account_defaults() -> None:
    client = CdekClient("TEST")

    assert client.base_url == "https://api.edu.cdek.ru/v2/"
    assert client.account == "wqGwiQx0gg8mLtiEKsUinjVSICCjtTEP"
    assert client.secure == "RmAmgvSgSl1yirlz9QupbzOJVqhCxcP5"
    assert client.account_type == "TEST"
    assert client.timeout == 5.0


def test_client_initializes_combat_account() -> None:
    client = CdekClient("login", secure="secret", timeout=10)

    assert client.base_url == "https://api.cdek.ru/v2/"
    assert client.account == "login"
    assert client.secure == "secret"
    assert client.account_type == "COMBAT"
    assert client.timeout == 10


def test_set_memory_stores_values_and_returns_self() -> None:
    client = CdekClient("login", secure="secret")
    memory = {"expires_in": 1, "access_token": "abc"}

    def callback(value: dict[str, str]) -> None:
        del value  # избегаем предупреждений

    returned = client.set_memory(memory, callback)

    assert returned is client
    assert client.memory is memory
    assert client.memory_save_callback is callback


@pytest.mark.parametrize(
    ("attribute", "expected_cls"),
    [
        ("currency", CurrencyApp),
        ("office", OfficeApp),
        ("order", OrderApp),
        ("location", LocationApp),
        ("tariff", TariffApp),
        ("barcode", BarcodeApp),
        ("invoice", InvoiceApp),
        ("payment", PaymentApp),
        ("agreement", AgreementApp),
        ("intake", IntakeApp),
        ("check", CheckApp),
        ("webhook", WebhookApp),
    ],
)
def test_client_app_properties(attribute: str, expected_cls: type) -> None:
    client = CdekClient("login", secure="secret")

    app = getattr(client, attribute)

    assert isinstance(app, expected_cls)
    assert app.client is client  # type: ignore


def test_test_account_exposes_all_apps_in_test_mode(test_client: CdekClient) -> None:
    expected_apps = [
        test_client.currency,
        test_client.office,
        test_client.order,
        test_client.location,
        test_client.tariff,
        test_client.barcode,
        test_client.invoice,
        test_client.payment,
        test_client.agreement,
        test_client.intake,
        test_client.check,
        test_client.webhook,
    ]

    for app in expected_apps:
        assert app.client.account_type == "TEST"
        assert app.client.base_url == "https://api.edu.cdek.ru/v2/"



from __future__ import annotations

from pydantic import BaseModel, Field

from ..models import (
    Dimensions,
    Error,
    FullLocation,
    Image,
    Phone,
    WarningModel,
    WorkTime,
    WorkTimeException,
)


class OfficeLocation(FullLocation):
    """Модель ответа локации офиса"""

    code: int = Field(alias="city_code")
    region: str | None = Field(default=None)  # type: ignore
    city: str | None = Field(default=None)  # type: ignore


class OfficeResponse(BaseModel):
    """Модель ответа офиса"""

    code: str = Field(..., description="Код ПВЗ")
    uuid: str = Field(..., description="Идентификатор офиса в ИС СДЭК")
    address_comment: str | None = Field(None, description="Описание местоположения")
    nearest_station: str | None = Field(
        None, description="Ближайшая станция/остановка транспорта"
    )
    nearest_metro_station: str | None = Field(
        None, description="Ближайшая станция метро"
    )
    work_time: str = Field(
        ..., description="Режим работы, строка вида «пн-пт 9-18, сб 9-16»"
    )
    phones: list[Phone] | None = Field(default=None, description="Телефоны офиса")
    email: str | None = Field(None, description="Адрес электронной почты")
    note: str | None = Field(None, description="Примечание по ПВЗ")
    type: str | None = Field(
        None, description="Тип ПВЗ. PVZ — склад СДЭК, POSTAMAT — постамат СДЭК"
    )
    owner_code: str = Field(..., description="Принадлежность офиса компании")
    take_only: bool = Field(
        ..., description="Является ли офис пунктом выдачи или осуществляет приём грузов"
    )
    is_handout: bool = Field(..., description="Является пунктом выдачи")
    is_reception: bool = Field(..., description="Является пунктом приёма")
    is_dressing_room: bool = Field(..., description="Есть ли примерочная")
    is_marketplace: bool | None = Field(
        None, description='Офис для доставки заказов "До маркетплейса"'
    )
    is_ltl: bool | None = Field(
        None, description="Работает ли офис с LTL (сборный груз)"
    )
    have_cashless: bool = Field(..., description="Есть безналичный расчет")
    have_cash: bool = Field(..., description="Есть приём наличных")
    have_fast_payment_system: bool = Field(
        ..., description="Есть безналичный расчёт по СБП"
    )
    allowed_cod: bool = Field(..., description="Разрешен наложенный платеж в ПВЗ")
    site: str | None = Field(None, description="Ссылка на данный офис на сайте СДЭК")
    office_image_list: list[Image] | None = Field(
        None, description="Все фото офиса (кроме фото как доехать)"
    )
    work_time_list: list[WorkTime] = Field(..., description="График работы на неделю")
    work_time_exception_list: list[WorkTimeException] = Field(
        ..., description="Исключения из графика работы"
    )
    weight_min: float | None = Field(
        None, description="Минимальный вес (в кг.), принимаемый в ПВЗ (> WeightMin)"
    )
    weight_max: float | None = Field(
        None, description="Максимальный вес (в кг.), принимаемый в ПВЗ (<= WeightMax)"
    )
    dimensions: list[Dimensions] | None = Field(None, description="Размеры ПВЗ")
    errors: list[Error] | None = Field(None, description="Список ошибок")
    warnings: list[WarningModel] | None = Field(
        None, description="Список предупреждений"
    )
    location: OfficeLocation = Field(..., description="Информация об офисе")
    distance: int | None = Field(
        None, description="Расстояние до точки, поиск ближайшего ПВЗ"
    )
    fulfillment: bool | None = Field(
        None, description='Работает ли офис с "Фулфилмент. Приход"'
    )



from __future__ import annotations

from pydantic import BaseModel, Field


class Dimensions(BaseModel):
    width: int = Field(..., description="Ширина (см)")
    height: int = Field(..., description="Высота (см)")
    depth: int = Field(..., description="Глубина (см)")



from .requests import (
    CalcAdditionalService,
    CalculatorLocation,
    TariffCodeRequest,
    TariffListRequest,
)
from .responses import (
    AvailableTariff,
    DeliveryDateRange,
    DeliveryMode,
    Services,
    TariffAvailableResponse,
    TariffListItem,
    TariffListResponse,
    TariffResponse,
)
from .tariff import TariffApp

__all__ = [
    "TariffApp",
    "TariffCodeRequest",
    "TariffListRequest",
    "TariffListResponse",
    "TariffResponse",
    "TariffAvailableResponse",
    "CalculatorLocation",
    "DeliveryDateRange",
    "TariffListItem",
    "Services",
    "CalcAdditionalService",
    "DeliveryMode",
    "AvailableTariff",
]



from __future__ import annotations

from pydantic import Field

from ..models.order import Order, UpdateOrder


class OrderRequest(Order):
    """Модель для запроса на создание заказа."""

    print: str | None = Field(None, description="Тип печатной формы")
    widget_token: str | None = Field(None, description="Токен CMS")


class OrderUpdateRequest(UpdateOrder):
    """Изменение заказа"""

    uuid: str | None = Field(None, description="Идентификатор заказа в ИС СДЭК")
    cdek_number: int | None = Field(None, description="Номер заказа в ИС СДЭК")



from __future__ import annotations

from datetime import date as Date
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer

from ..models.status import Status
from .requests import RegisterDeliveryRequest


class AvailableDeliveryInterval(BaseModel):
    start_time: str = Field(..., description="Время начала интервала доставки")
    end_time: str = Field(..., description="Время окончания интервала доставки")


class AvailableDeliveryIntervalsInfo(BaseModel):
    date: Date = Field(
        ..., description="Дата доступного интервала для доставки (формат yyyy-MM-dd)"
    )
    time_intervals: list[AvailableDeliveryInterval] = Field(
        ..., description="Временные интервалы для доставки"
    )

    @field_serializer("date")
    def serialize_date(self, date: Date) -> str:
        """Представить дату интервала доставки в формате YYYY-MM-DD"""
        return date.strftime("%Y-%m-%d")


class AvailableDeliveryIntervalsResponse(BaseModel):
    date_intervals: list[AvailableDeliveryIntervalsInfo] = Field(
        ..., description="Доступные интервалы доставки"
    )


class ScheduleInfoEntity(RegisterDeliveryRequest):
    uuid: UUID = Field(..., description="Идентификатор договоренности о доставке")
    statuses: list[Status] = Field(..., description="Статусы договоренности о доставке")
    source: str | None = Field(
        None,
        description="Источник согласования",
    )


class AgreementInfoResponse(BaseModel):
    entity: ScheduleInfoEntity | None = Field(
        default=None, description="Договорённость о доставке"
    )



from typing import Any

from ..app import App
from ..models.entity_response import EntityResponse
from .requests import DeliveryIntervalRequest, RegisterDeliveryRequest
from .responses import AgreementInfoResponse, AvailableDeliveryIntervalsResponse


class AgreementApp(App):
    """Класс для работы с договоренностями о доставке"""

    def _get_interval(
        self, params: dict[str, Any]
    ) -> AvailableDeliveryIntervalsResponse:
        """Получение интервалов доставки

        Args:
            params (Any): параметры для получения интервалов доставки

        Returns:
            AvailableDeliveryIntervalsResponse: объект с информацией
            о интервалах доставки
        """
        response = self._get("delivery/intervals", params)
        return AvailableDeliveryIntervalsResponse.model_validate(response)

    def get_interval_number(
        self, cdek_number: str
    ) -> AvailableDeliveryIntervalsResponse:
        """Получить интервалы доставки по номеру заказа

        Args:
            cdek_number (str): номер заказа

        Returns:
            AvailableDeliveryIntervalsResponse: объект с информацией
            о интервалах доставки
        """
        return self._get_interval({"cdek_number": cdek_number})

    def get_interval_uuid(self, uuid: str) -> AvailableDeliveryIntervalsResponse:
        """Получение интервалов доставки по UUID заказа

        Args:
            uuid (str): UUID заказа

        Returns:
            AvailableDeliveryIntervalsResponse: объект с информацией
            о интервалах доставки
        """
        return self._get_interval({"order_uuid": uuid})

    def create(self, agreement: RegisterDeliveryRequest) -> EntityResponse:
        """Регистрация договоренности о доставке

        Args:
            agreement (RegisterDeliveryRequest): объект с информацией
             о договоренности о доставке

        Returns:
            EntityResponse: объект с информацией о регистрации договоренности о доставке

        Raises:
            ValueError: если agreement не является объектом RegisterDeliveryRequest
        """
        if not isinstance(agreement, RegisterDeliveryRequest):
            raise ValueError("agreement must be a RegisterDeliveryRequest")
        response = self._post("delivery", json=agreement)
        return EntityResponse.model_validate(response)

    def get(self, uuid: str) -> AgreementInfoResponse:
        """Получение договоренностей для курьера

        Args:
            uuid (str): UUID договора

        Returns:
            AgreementInfoResponse: объект с информацией о договорах
        """
        response = self._get(f"delivery/{uuid}")
        return AgreementInfoResponse.model_validate(response)

    def get_intervals_before_create_order(
        self, request: "DeliveryIntervalRequest"
    ) -> AvailableDeliveryIntervalsResponse:
        """Получение интервалов доставки до создания заказа

        Args:
            request (DeliveryIntervalRequest): объект с информацией
            о запросе на получение интервалов доставки до создания заказа

        Returns:
            AvailableDeliveryIntervalsResponse: объект с информацией о интервалах
                доставки до создания заказа

        Raises:
            ValueError: если request не является объектом DeliveryIntervalRequest
        """
        if not isinstance(request, DeliveryIntervalRequest):
            raise ValueError("request must be a DeliveryIntervalRequest")
        response = self._post("delivery/estimatedIntervals", json=request)
        return AvailableDeliveryIntervalsResponse.model_validate(response)



from . import (
    agreement,
    check,
    form,
    intake,
    location,
    office,
    order,
    payment,
    tariff,
    webhook,
)

__all__ = [
    "location",
    "office",
    "payment",
    "tariff",
    "webhook",
    "agreement",
    "form",
    "check",
    "intake",
    "order",
]



from __future__ import annotations

from ..app import App
from .filters import (
    CityFilter,
    CityListFilter,
    CoordinatesFilter,
    RegionFilter,
    ZipFilter,
)
from .responses import (
    CitiesResponse,
    CityResponse,
    CoordinatesResponse,
    RegionResponse,
    ZipResponse,
)


class LocationApp(App):
    """Получение списка локаций"""

    def city(self, filter_params: CityFilter) -> CityResponse | None:
        """
        Подбор локации по названию города
        Метод позволяет получать подсказки по подбору населенного пункта
        по его наименованию.

        Args:
            filter_params (CityFilter): Фильтр для подбора города

        Raises:
            ValueError: Если filter_params не является объектом CityFilter

        Returns:
            Optional[CityResponse]: Объект CityResponse с информацией о найденном городе
            или None, если город не найден
        """
        # Обрабатываем filter_params как словарь
        if filter_params is not None and not isinstance(filter_params, CityFilter):
            raise ValueError("filter_params must be a CityFilter instance")
        response = self._get("location/suggest/cities", params=filter_params)
        return CityResponse.model_validate(response) if response else None

    def regions(
        self, filter_params: RegionFilter | None = None
    ) -> list[RegionResponse]:
        """
        Получение списка регионов
        Метод предназначен для получения детальной информации о регионах.

        Args:
            filter_params (RegionFilter): Фильтр для получения списка регионов

        Raises:
            ValueError: Если filter_params не является объектом RegionFilter

        Returns:
            list[RegionResponse]: Список объектов RegionResponse
            с информацией о регионах
        """
        if filter_params is not None and not isinstance(filter_params, RegionFilter):
            raise ValueError("filter_params must be a RegionFilter instance")
        response = self._get("location/regions", params=filter_params)
        # Здесь должен быть импорт и создание RegionsResponse объектов
        return [RegionResponse.model_validate(region) for region in response]

    def zip(self, filter_params: ZipFilter | None = None) -> list[ZipResponse]:
        """
        Получение почтовых индексов города
        Метод предназначен для получения списка почтовых индексов.

        Args:
            filter_params (ZipFilter): Фильтр для получения списка почтовых индексов

        Raises:
            ValueError: Если filter_params не является объектом ZipFilter

        Returns:
            list[ZipResponse]: Список объектов ZipResponse с информацией
            о почтовых индексах
        """
        if filter_params is not None and not isinstance(filter_params, ZipFilter):
            raise ValueError("filter_params must be a ZipFilter instance")
        response = self._get("location/postalcodes", params=filter_params)
        return [ZipResponse.model_validate(zip_code) for zip_code in response]

    def coordinates(
        self, filter_params: CoordinatesFilter | None = None
    ) -> list[CoordinatesResponse]:
        """
        Получение локации по координатам
        Метод позволяет определить локацию по переданным в запросе координатам

        Args:
            filter_params (CoordinatesFilter): Фильтр
            для получения локации по координатам

        Raises:
            ValueError: Если filter_params не является объектом CoordinatesFilter

        Returns:
            list[CoordinatesResponse]: Список объектов CoordinatesResponse
            с информацией о локации
        """
        if filter_params is not None and not isinstance(
            filter_params, CoordinatesFilter
        ):
            raise ValueError("filter_params must be a CoordinatesFilter instance")
        response = self._get("location/coordinates", params=filter_params)
        return [
            CoordinatesResponse.model_validate(coordinates) for coordinates in response
        ]

    def cities(
        self, filter_params: CityListFilter | None = None
    ) -> list[CitiesResponse]:
        """
        Получение списка населенных пунктов
        Метод предназначен для получения детальной информации о населенных пунктах

        Args:
            filter_params (CityListFilter): Фильтр для получения списка городов

        Raises:
            ValueError: Если filter_params не является объектом CityListFilter

        Returns:
            list[CitiesResponse]: Список объектов CitiesResponse с информацией о городах
        """
        if filter_params is not None and not isinstance(filter_params, CityListFilter):
            raise ValueError("filter_params must be a CityListFilter instance")
        response = self._get("location/cities", params=filter_params)
        return [CitiesResponse.model_validate(city) for city in response]



from __future__ import annotations

from uuid import UUID

from pydantic import BaseModel, Field


class City(BaseModel):
    code: int = Field(..., description="Код населенного пункта СДЭК")
    city_uuid: UUID = Field(
        ..., description="Идентификатор населенного пункта в ИС СДЭК"
    )
    city: str = Field(..., description="Название населенного пункта")
    fias_guid: UUID | None = Field(
        None, description="Уникальный идентификатор ФИАС населенного пункта"
    )


class Region(BaseModel):
    region: str = Field(..., max_length=255, description="Название региона")
    region_code: int | None = Field(None, description="Код региона СДЭК")
    kladr_region_code: str | None = Field(
        None, description="Код КЛАДР региона населенного пункта"
    )
    sub_region: str | None = Field(
        None, description="Название района региона населенного пункта"
    )


class Zip(BaseModel):
    postal_code: str | None = Field(None, description="Почтовые индексы города")


class Country(BaseModel):
    country_code: str | None = Field(
        None, max_length=2, description="Код страны в формате ISO_3166-1_alpha-2"
    )
    country: str | None = Field(
        None, max_length=255, description="Название страны региона"
    )


class Coordinates(BaseModel):
    longitude: float | None = Field(None, description="Долгота населенного пункта")
    latitude: float | None = Field(None, description="Широта населенного пункта")


class Address(BaseModel):
    address: str | None = Field(None, description="Адрес населенного пункта")
    address_full: str | None = Field(
        None, description="Полный адрес с указанием страны, региона, города, и т.д."
    )


class FullLocation(City, Country, Region, Coordinates, Address, Zip):
    pass


class OrderLocation(City, Country, Region, Coordinates, Zip):
    code: int | None = Field(default=None)  # type: ignore
    city_uuid: UUID | None = Field(default=None)  # type: ignore
    region: str | None = Field(default=None)  # type: ignore
    city: str | None = Field(default=None)  # type: ignore
    time_zone: str | None = Field(None, description="Часовой пояс населенного пункта")
    address: str = Field(..., description="Строка адреса")



from __future__ import annotations

from typing import Literal
from uuid import UUID

from pydantic import BaseModel, Field


class OfficeFilter(BaseModel):
    """Фильтр для ПВЗ"""

    code: str | None = Field(None, max_length=255, description="Код ПВЗ")
    type: Literal["PVZ", "POSTAMAT", "ALL"] = Field("ALL", description="Тип офиса")
    postal_code: str | None = Field(
        None,
        max_length=255,
        description="Почтовый индекс города, для которого необходим список офисов",
    )
    city_code: int | None = Field(
        None,
        description='Код населенного пункта СДЭК (метод "Список населенных пунктов")',
    )
    country_code: str | None = Field(
        None,
        max_length=2,
        description="Код страны в формате ISO_3166-1_alpha-2",
    )
    region_code: int | None = Field(None, description="Код региона СДЭК")
    have_cashless: bool | None = Field(None, description="Наличие терминала оплаты")
    have_cash: bool | None = Field(None, description="Есть прием наличных")
    is_dressing_room: bool | None = Field(None, description="Наличие примерочной")
    weight_max: int | None = Field(
        None, gt=0, description="Максимальный вес в кг, который может принять офис"
    )
    weight_min: int | None = Field(
        None, gt=0, description="Минимальный вес в кг, который может принять офис"
    )
    lang: str = Field("rus", description="Локализация офиса")
    take_only: bool | None = Field(
        None, description="Является ли офис только пунктом выдачи"
    )
    is_handout: bool | None = Field(None, description="Является пунктом выдачи")
    is_reception: bool | None = Field(None, description="Есть ли в офисе приём заказов")
    is_marketplace: bool | None = Field(
        None, description='Офис для доставки "До маркетплейса"'
    )
    is_ltl: bool | None = Field(
        None, description="Работает ли офис с LTL (сборный груз)"
    )
    fulfillment: bool | None = Field(None, description="Офис с зоной фулфилмента")
    fias_guid: UUID | None = Field(None, description="Код города ФИАС")
    size: int | None = Field(
        None, ge=0, description="Ограничение выборки результата (размер страницы)"
    )
    page: int | None = Field(
        None, ge=0, description="Номер страницы выборки результата"
    )



from __future__ import annotations

from .contact import Contact, ContragentType, Passport, Tin
from .dimensions import Dimensions
from .entity_response import EntityResponse, RootEntity
from .error import Error
from .image import Image
from .intakes import IntakeLocation, Intakes
from .item import Item
from .location import (
    Address,
    City,
    Coordinates,
    Country,
    FullLocation,
    OrderLocation,
    Region,
    Zip,
)
from .money import Money, Vat
from .order import (
    AccompanyingWaybill,
    AdditionalService,
    Call,
    DeliveryCostThreshold,
    DeliveryDetail,
    DeliveryProblem,
    DeliveryRecipientCost,
    FailedCall,
    Order,
    OrderInfo,
    OrderStatus,
    RescheduledCall,
    UpdateOrder,
)
from .package import CalcPackage, Package
from .payment_info import PaymentInfo
from .phone import Phone
from .print import PrintForm, PrintOrder, PrintType
from .related_entity import RelatedEntity, RelatedEntityType
from .request import Request
from .seller import Seller
from .status import Status
from .warning import WarningModel
from .work_time import WorkTime, WorkTimeException

__all__ = [
    "EntityResponse",
    "RootEntity",
    "RelatedEntity",
    "Request",
    "Phone",
    "Contact",
    "Image",
    "WorkTime",
    "WorkTimeException",
    "Dimensions",
    "Error",
    "WarningModel",
    "Address",
    "City",
    "Coordinates",
    "Country",
    "Region",
    "Zip",
    "ContragentType",
    "Passport",
    "Tin",
    "CalcPackage",
    "Package",
    "FullLocation",
    "Seller",
    "Money",
    "Vat",
    "PaymentInfo",
    "PrintType",
    "PrintForm",
    "PrintOrder",
    "OrderStatus",
    "DeliveryDetail",
    "DeliveryProblem",
    "FailedCall",
    "RescheduledCall",
    "Call",
    "DeliveryRecipientCost",
    "DeliveryCostThreshold",
    "AdditionalService",
    "UpdateOrder",
    "Order",
    "OrderInfo",
    "Status",
    "AccompanyingWaybill",
    "OrderLocation",
    "Item",
    "RelatedEntityType",
    "IntakeLocation",
    "Intakes",
]



from __future__ import annotations

from datetime import date as Date
from datetime import datetime
from typing import Any

from pydantic import BaseModel, Field, field_serializer

from .contact import Contact
from .location import OrderLocation
from .money import Money, Vat
from .package import Package
from .payment_info import PaymentInfo
from .seller import Seller


class AccompanyingWaybill(BaseModel):
    client_name: str = Field(
        ..., description="Наименование юрлица клиента, создающего СНТ"
    )
    flight_number: str | None = Field(None, description="Номер рейса")
    air_waybill_numbers: list[str] | None = Field(
        None, description="Накладные перевозчика для авиарейса"
    )
    vehicle_numbers: list[str] | None = Field(None, description="Номера автомобилей")
    vehicle_driver: str | None = Field(None, description="Водитель автомобиля")
    planned_departure_date_time: datetime | None = Field(
        None, description="Планируемая дата отправления"
    )

    @field_serializer("planned_departure_date_time")
    def serialize_planned_departure_date_time(
        self, planned_departure_date_time: datetime
    ) -> str:
        """Преобразовать дату отправления СНТ в строку ISO 8601"""
        return planned_departure_date_time.strftime("%Y-%m-%dT%H:%M:%S")


class OrderStatus(BaseModel):
    """Статус заказа."""

    code: str | None = Field(None, description="Код статуса")
    name: str | None = Field(None, description="Название статуса")
    date_time: datetime | None = Field(
        None, description="Дата и время установки статуса"
    )
    reason_code: str | None = Field(None, description="Дополнительный код статуса")
    city: str | None = Field(
        None, description="Наименование места возникновения статуса"
    )
    city_uuid: str | None = Field(
        None, description="Идентификатор места (города) возникновения статуса"
    )
    deleted: bool | None = Field(None, description="Признак удаления статуса")

    @field_serializer("date_time")
    def serialize_date_time(self, date_time: datetime) -> str:
        """Вернуть время установки статуса в формате ISO 8601"""
        return date_time.strftime("%Y-%m-%dT%H:%M:%S")


class DeliveryDetail(BaseModel):
    date: Date | None = Field(None, description="Дата доставки")
    recipient_name: str | None = Field(None, description="Получатель при доставке")
    payment_sum: float | None = Field(None, description="Сумма наложенного платежа")
    delivery_sum: float | None = Field(None, description="Сумма доставки")
    total_sum: float = Field(..., description="Общая сумма")
    payment_info: list[PaymentInfo] | None = Field(
        None, description="Информация о платеже"
    )
    delivery_vat_rate: float | None = Field(None, description="Ставка НДС для доставки")
    delivery_vat_sum: float | None = Field(None, description="Сумма НДС для доставки")
    delivery_discount_percent: float | None = Field(
        None, description="Процент скидки для доставки"
    )
    delivery_discount_sum: float | None = Field(
        None, description="Сумма скидки для доставки"
    )

    @field_serializer("date")
    def serialize_date(self, date: Date) -> str:
        """Представить дату доставки в формате YYYY-MM-DD"""
        return date.strftime("%Y-%m-%d")


class DeliveryProblem(BaseModel):
    code: str | None = Field(None, description="Код проблемы")
    create_date: datetime | None = Field(None, description="Дата создания проблемы")

    @field_serializer("create_date")
    def serialize_create_date(self, create_date: datetime) -> str:
        """Вернуть дату создания проблемы в формате ISO 8601"""
        return create_date.strftime("%Y-%m-%dT%H:%M:%S")


class FailedCall(BaseModel):
    date_time: datetime = Field(..., description="Дата и время недозвона")
    reason_code: int = Field(..., description="Причина недозвона")

    @field_serializer("date_time")
    def serialize_date_time(self, date_time: datetime) -> str:
        """Представить дату и время недозвона в формате ISO 8601"""
        return date_time.strftime("%Y-%m-%dT%H:%M:%S")


class RescheduledCall(BaseModel):
    date_time: datetime = Field(
        ..., description="Дата и время создания переноса прозвона"
    )
    date_next: Date = Field(..., description="Дата переноса прозвона")
    time_next: str = Field(..., description="Время переноса прозвона")
    comment: str | None = Field(None, description="Комментарий к переносу прозвона")

    @field_serializer("date_time")
    def serialize_date_time(self, date_time: datetime) -> str:
        """Представить дату создания переноса прозвона в формате ISO 8601"""
        return date_time.strftime("%Y-%m-%dT%H:%M:%S")

    @field_serializer("date_next")
    def serialize_date_next(self, date_next: Date) -> str:
        """Вернуть дату следующего прозвона в формате YYYY-MM-DD"""
        return date_next.strftime("%Y-%m-%d")


class Call(BaseModel):
    failed_calls: list[FailedCall] | None = Field(
        None, description="Информация о неуспешных прозвонах (недозвонах)"
    )
    rescheduled_calls: list[RescheduledCall] | None = Field(
        None, description="Информация о переносах прозвона"
    )


class DeliveryRecipientCost(Money):
    value: float | None = Field(None, description="Сумма платежа, включая НДС")


class DeliveryCostThreshold(Vat):
    threshold: int | None = Field(None, description="Порог стоимости товара")
    sum: float | None = Field(
        None,
        description="Доп. сбор за доставку товаров",
    )


class AdditionalService(BaseModel):
    code: str = Field(..., description="Тип дополнительной услуги")
    parameter: str | None = Field(None, description="Параметр дополнительной услуги")


class UpdateOrder(BaseModel):
    type: int | None = Field(None, description="Тип заказа")
    number: str | None = Field(None, description="Номер заказа в ИС Клиента")
    accompanying_number: str | None = Field(
        None, description="Номер сопроводительной накладной на товар (СНТ)"
    )
    tariff_code: int | None = Field(None, description="Код тарифа")
    comment: str | None = Field(None, description="Комментарий к заказу")
    shipment_point: str | None = Field(None, description="Код ПВЗ СДЭК")
    delivery_point: str | None = Field(None, description="Код ПВЗ СДЭК")
    delivery_recipient_cost: DeliveryRecipientCost | None = Field(
        None, description="Стоимость доставки"
    )
    delivery_recipient_cost_adv: list[DeliveryCostThreshold] | None = Field(
        None, description="Пороговая стоимость доставки"
    )
    sender: Contact | None = Field(None, description="Отправитель")
    seller: Seller | None = Field(None, description="Реквизиты истинного продавца")
    recipient: Contact = Field(..., description="Получатель")
    from_location: OrderLocation | None = Field(None, description="Адрес отправления")
    to_location: OrderLocation | None = Field(None, description="Адрес доставки")
    services: list[AdditionalService] | None = Field(
        None, description="Дополнительные услуги"
    )
    packages: list[Package] | None = Field(None, description="Упаковки")
    has_reverse_order: bool | None = Field(
        None, description="Признак необходимости создания реверсного заказа"
    )

    @classmethod
    def location_init(cls, **kwargs: Any) -> OrderLocation:
        """Создать объект адреса заказа с указанными полями"""
        return OrderLocation(**kwargs)

    @classmethod
    def seller_init(cls, **kwargs: Any) -> Seller:
        """Создать объект продавца по переданным параметрам"""
        return Seller(**kwargs)

    @classmethod
    def contact_init(cls, **kwargs: Any) -> Contact:
        """Создать объект контакта отправителя или получателя"""
        return Contact(**kwargs)

    def set_seller(self, seller: Seller) -> Order:
        """Установить продавца для текущего заказа"""
        self.seller = seller
        return self

    def set_from_location(self, location: OrderLocation) -> Order:
        """Задать адрес отправления"""
        self.from_location = location
        return self

    def set_to_location(self, location: OrderLocation) -> Order:
        """Задать адрес доставки"""
        self.to_location = location
        return self

    def set_shipment_point(self, code: str) -> Order:
        """Задать код ПВЗ СДЭК"""
        self.shipment_point = code
        return self

    def set_delivery_point(self, code: str) -> Order:
        """Задать код ПВЗ СДЭК"""
        self.delivery_point = code
        return self

    def set_contact(self, sender: Contact) -> Order:
        """Сохранить контактные данные отправителя"""
        self.sender = sender
        return self

    def set_recipient(self, recipient: Contact) -> Order:
        """Сохранить контактные данные получателя"""
        self.recipient = recipient
        return self

    @classmethod
    def package_init(cls, **kwargs: Any) -> Package:
        """Создать объект упаковки по переданным параметрам"""
        return Package(**kwargs)


class Order(UpdateOrder):
    additional_order_types: list[int] | None = Field(
        None, description="Дополнительные типы заказа"
    )
    tariff_code: int = Field(..., description="Код тарифа")
    date_invoice: Date | None = Field(None, description="Дата инвойса")
    shipper_name: str | None = Field(None, description="Грузоотправитель")
    shipper_address: str | None = Field(None, description="Адрес грузоотправителя")
    services: list[AdditionalService] | None = Field(
        None, description="Дополнительные услуги"
    )
    packages: list[Package] = Field(default_factory=list, description="Упаковки")
    is_client_return: bool | None = Field(
        None, description="Признак клиентского возврата"
    )

    @field_serializer("date_invoice")
    def serialize_date_invoice(self, date_invoice: Date) -> str:
        """Вернуть дату инвойса в формате YYYY-MM-DD"""
        return date_invoice.strftime("%Y-%m-%d")

    def add_package(self, package: Package) -> Order:
        """Добавить упаковку в список пакетов заказа"""
        self.packages.append(package)
        return self


class OrderInfo(BaseModel):
    cdek_number: int | None = Field(None, description="Номер заказа")
    order_uuid: str | None = Field(None, description="Идентификатор заказа")



from __future__ import annotations

from pydantic import BaseModel, Field


class Image(BaseModel):
    number: int | None = Field(0, description="Номер фото")
    url: str = Field(..., max_length=255, description="Ссылка на фото")



from __future__ import annotations

from datetime import date as Date
from uuid import UUID

from pydantic import BaseModel, Field

from ..models.entity_response import EntityResponse
from ..models.error import Error
from ..models.intakes import IntakeLocation, Intakes
from ..models.status import Status
from ..models.warning import WarningModel


class IntakesResponse(Intakes):
    pass


class IntakePackage(BaseModel):
    package_id: UUID | None = Field(
        None,
        description="Уникальный номер упаковки в ИС СДЭК",
    )
    weight: int | None = Field(
        None,
        description="Общий вес упаковки (в граммах)",
    )
    length: int | None = Field(
        None,
        description="Длина упаковки (в сантиметрах)",
    )
    width: int | None = Field(
        None,
        description="Ширина упаковки (в сантиметрах)",
    )
    height: int | None = Field(
        None,
        description="Высота упаковки (в сантиметрах)",
    )


class IntakesEntity(BaseModel):
    order_uuid: UUID = Field(..., alias="uuid")
    intake_number: str | None = Field(
        None, max_length=255, description="Номер заявки в системе СДЭК"
    )
    to_location: IntakeLocation | None = Field(None, description="Место доставки")
    statuses: list[Status] | None = Field(None, description="Статусы заявки")
    packages: list[IntakePackage] | None = Field(
        None, description="Список упаковок заявки"
    )
    contragent_uuid: UUID | None = Field(None, description="Идентификатор контрагента")


class IntakeEntityResponse(EntityResponse):
    entity: IntakesEntity | None = None


class IntakeDateResponse(BaseModel):
    date: list[Date] = Field(..., description="Доступные даты для забора курьером")
    all_days: bool | None = Field(None, description="Все дни")
    errors: list[Error] | None = Field(None, description="Ошибки")
    warnings: list[WarningModel] | None = Field(None, description="Предупреждения")



from . import constants
from .client import CdekClient
from .exceptions import CdekAuthException, CdekException, CdekRequestException

__version__ = "1.1.10"
__author__ = "CDEK Python SDK"
__license__ = "Apache-2.0"

__all__ = [
    "CdekClient",
    "CdekException",
    "CdekAuthException",
    "CdekRequestException",
    "constants",
]



from __future__ import annotations

from uuid import UUID

from pydantic import BaseModel, Field

from .related_entity import RelatedEntity
from .request import Request


class RootEntity(BaseModel):
    uuid: UUID = Field(..., description="Идентификатор сущности в ИС СДЭК")


class EntityResponse(BaseModel):
    """Модель ответа о сущности."""

    entity: RootEntity | None = Field(
        default=None, description="Идентификатор сущности в ИС СДЭК"
    )
    requests: list[Request] | None = None
    related_entities: list[RelatedEntity] | None = None

    def get_entry_uuid(self) -> str | None:
        """Получить UUID сущности."""
        if self.entity is None:
            return None
        if hasattr(self.entity, "uuid"):
            return str(self.entity.uuid)
        return None

    def get_entity(self) -> RootEntity | None:
        """Получить данные сущности."""
        if self.entity is None:
            return None
        return self.entity

    def get_requests(self) -> list[Request]:
        """Получить список запросов."""
        if self.requests is None:
            return []
        return self.requests



from __future__ import annotations

from uuid import UUID

from pydantic import BaseModel, Field

from ..models.location import City, Coordinates, Country, Region


class CityResponse(BaseModel):
    code: int = Field(..., description="Код населенного пункта СДЭК")
    city_uuid: UUID = Field(..., description="Идентификатор населенного пункта СДЭК")
    full_name: str = Field(
        ...,
        max_length=255,
        description="Наименование населенного пункта СДЭК",
    )
    country_code: str = Field(
        ..., max_length=255, description="Код страны в формате ISO_3166-1_alpha-2"
    )


class RegionResponse(Country, Region):
    pass


class ZipResponse(BaseModel):
    code: int = Field(
        ..., description="Код города, которому принадлежат почтовые индексы"
    )
    postal_code: list[str] = Field(..., description="Почтовые индексы города")


class CoordinatesResponse(City):
    pass


class CitiesResponse(City, Country, Region, Coordinates):
    region: str | None = Field(
        default=None, max_length=255, description="Название региона"
    )  # type: ignore
    time_zone: str | None = Field(None, description="Часовой пояс населенного пункта")
    payment_limit: float | None = Field(
        ..., description="Ограничение на сумму наложенного платежа в населенном пункте"
    )



from ..app import App
from ..models.entity_response import EntityResponse
from .requests import PrintBarcodeRequest
from .responses import PrintBarcodeResponse


class BarcodeApp(App):
    """Класс для работы с ШК-местами к заказу"""

    def set(self, barcode: PrintBarcodeRequest) -> EntityResponse:
        """Запрос на формирование ШК-места к заказу

        Args:
            barcode (PrintBarcodeRequest): объект с информацией о запросе
                на формирование ШК-места к заказу

        Returns:
            EntityResponse: объект с информацией о формировании ШК-места к заказу

        Raises:
            ValueError: если barcode не является объектом PrintBarcodeRequest
        """
        if not isinstance(barcode, PrintBarcodeRequest):
            raise ValueError("barcode must be a PrintBarcodeRequest")
        response = self._post("print/barcodes", json=barcode)
        return EntityResponse.model_validate(response)

    def get(self, uuid: str) -> PrintBarcodeResponse:
        """Получение сущности ШК к заказу

        Args:
            uuid (str): идентификатор ШК-места к заказу

        Returns:
            PrintBarcodeResponse: объект с информацией о ШК-месте к заказу
        """
        response = self._get(f"print/barcodes/{uuid}")
        return PrintBarcodeResponse.model_validate(response)

    def get_pdf(self, uuid: str) -> bytes:
        """Скачивание готового ШК

        Args:
            uuid (str): идентификатор ШК-места к заказу

        Returns:
            bytes: содержимое PDF файла
        """
        return self._get(f"print/barcodes/{uuid}.pdf")



from __future__ import annotations

from pydantic import BaseModel, Field


class Vat(BaseModel):
    vat_sum: float | None = Field(None, description="Сумма НДС")
    vat_rate: int | None = Field(None, description="Ставка НДС")


class Money(Vat):
    """Модель для денежных сумм и НДС."""

    value: float | None = Field(None, description="Сумма платежа, включая НДС")



from __future__ import annotations

from datetime import datetime
from typing import Literal
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer

from ..models.entity_response import EntityResponse
from ..models.print import PrintForm, PrintType
from .requests import PrintBarcodeRequest


class PrintStatus(BaseModel):
    code: str = Field(..., description="Код статуса")
    name: str = Field(..., description="Название статуса")
    date_time: datetime = Field(..., description="Дата и время статуса")

    @field_serializer("date_time")
    def serialize_date_time(self, date_time: datetime) -> str:
        """Вернуть дату статуса печатной формы в формате ISO 8601"""
        return date_time.strftime("%Y-%m-%dT%H:%M:%S")


class PrintBarcodeEntity(PrintBarcodeRequest):
    uuid: UUID = Field(..., description="Идентификатор ШК места к заказу")
    url: str | None = Field(None, description="Ссылка на скачивание файла")
    lang: Literal["RUS", "ENG", "DEU", "ITA", "TUR", "CES", "KOR", "LIT", "LAV"] = (
        Field(default="RUS", description="Язык печати")  # type: ignore
    )
    statuses: list[PrintStatus] = Field(..., description="Статус файла")


class PrintBarcodeResponse(EntityResponse):
    entity: PrintBarcodeEntity | None = Field(
        default=None, description="ШК места к заказу"
    )


class WaybillResponse(PrintForm):
    uuid: UUID = Field(..., description="Идентификатор ШК места к заказу")
    type: PrintType | None = Field(None, description="Форма квитанции")
    url: str | None = Field(None, description="Ссылка на скачивание файла")
    statuses: list[PrintStatus] = Field(..., description="Статус файла")


class WaybillEntityResponse(EntityResponse):
    entity: WaybillResponse | None = Field(
        default=None, description="Квитанция к заказу"
    )



from __future__ import annotations

from pydantic import BaseModel, Field


class WarningModel(BaseModel):
    """Предупреждения"""

    code: str | None = Field(None, description="Код предупреждения")
    message: str | None = Field(None, description="Описание предупреждения")



from __future__ import annotations

from datetime import date as Date

from pydantic import BaseModel, Field, field_serializer


class CheckFilter(BaseModel):
    """Фильтр для получения информации о чеке"""

    order_uuid: str | None = Field(..., description="Идентификатор заказа")
    cdek_number: str | None = Field(..., description="Номер заказа СДЭК")
    date: Date | None = Field(
        ..., description="Дата создания чека в формате YYYY-MM-DD"
    )

    @field_serializer("date")
    def serialize_date(self, date: Date) -> str:
        """Преобразовать дату фильтра в строку YYYY-MM-DD"""
        return date.strftime("%Y-%m-%d")



from __future__ import annotations

from datetime import datetime
from typing import Literal
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer

from .error import Error
from .warning import WarningModel


class Request(BaseModel):
    """Модель ответа о запросе."""

    request_uuid: UUID | None = Field(
        None, description="Идентификатор запроса в ИС СДЭК"
    )
    type: Literal[
        "CREATE",
        "UPDATE",
        "DELETE",
        "AUTH",
        "GET",
        "CREATE_CLIENT_RETURN",
        "CREATE_REFUSAL",
    ] = Field(..., description="Тип запроса")
    date_time: datetime = Field(
        ..., description="Дата и время установки текущего состояния запроса"
    )
    state: Literal["ACCEPTED", "WAITING", "SUCCESSFUL", "INVALID"] = Field(
        ..., max_length=255, description="Текущее состояние запроса"
    )
    errors: list[Error] | None = Field(
        None, description="Ошибки, возникшие в ходе выполнения запроса"
    )
    warnings: list[WarningModel] | None = Field(
        None, description="Предупреждения, возникшие в ходе выполнения запроса"
    )

    def get_state(self) -> str | None:
        """Получить состояние запроса."""
        return self.state

    @field_serializer("date_time")
    def serialize_date_time(self, date_time: datetime) -> str:
        """Представить дату последнего изменения запроса в формате ISO"""
        return date_time.strftime("%Y-%m-%dT%H:%M:%S")



from .filters import (
    CityFilter,
    CityListFilter,
    CoordinatesFilter,
    RegionFilter,
    ZipFilter,
)
from .location import LocationApp
from .responses import (
    CitiesResponse,
    CityResponse,
    CoordinatesResponse,
    RegionResponse,
    ZipResponse,
)

__all__ = [
    "LocationApp",
    "CityFilter",
    "CityListFilter",
    "CoordinatesFilter",
    "RegionFilter",
    "ZipFilter",
    "CityResponse",
    "RegionResponse",
    "ZipResponse",
    "CoordinatesResponse",
    "CitiesResponse",
]



from .cdek_auth_exception import CdekAuthException
from .cdek_exception import CdekException
from .cdek_request_exception import CdekRequestException

__all__ = [
    "CdekException",
    "CdekAuthException",
    "CdekRequestException",
]



from __future__ import annotations

from uuid import UUID

from pydantic import BaseModel, Field

from ..models.entity_response import EntityResponse
from .enums import WebhookType


class WebhookResponse(BaseModel):
    """Модель ответа о списке webhooks."""

    uuid: UUID = Field(..., description="Идентификатор вебхука")
    type: WebhookType = Field(..., description="Тип вебхука")
    url: str = Field(..., description="URL, на который отправляется событие")


class WebookSetEntityResponse(EntityResponse):
    """Ответ на добавление подписки на webhook"""


class WebhookUUIDEntityResponse(EntityResponse):
    """Ответ на получение информации о подписке по UUID"""

    entity: WebhookResponse | None = Field(default=None, description="Webhook")


class WebhookDeleteEntityResponse(EntityResponse):
    """Ответ на удаление подписки на webhook"""



from ..app import App
from ..models.entity_response import EntityResponse
from .requests import PrintInvoiceRequest
from .responses import WaybillEntityResponse


class InvoiceApp(App):
    """Класс для работы с накладными к заказу"""

    def get(self, uuid: str) -> WaybillEntityResponse:
        """Получение сущности накладной к заказу

        Args:
            uuid (str): идентификатор накладной к заказу

        Returns:
            WaybillEntityResponse: объект с информацией о накладной к заказу
        """
        response = self._get(f"print/orders/{uuid}")
        return WaybillEntityResponse.model_validate(response)

    def get_pdf(self, uuid: str) -> bytes:
        """Получение PDF накладной к заказу

        Args:
            uuid (str): идентификатор накладной к заказу

        Returns:
            bytes: содержимое PDF файла
        """
        return self._get(f"print/orders/{uuid}.pdf")

    def set(self, invoice: PrintInvoiceRequest) -> EntityResponse:
        """Запрос на формирование накладной к заказу

        Args:
            invoice (PrintInvoiceRequest): объект с информацией о запросе
                на формирование накладной к заказу

        Returns:
            EntityResponse: объект с информацией о формировании накладной к заказу

        Raises:
            ValueError: если invoice не является объектом PrintInvoiceRequest
        """
        if not isinstance(invoice, PrintInvoiceRequest):
            raise ValueError("invoice must be a PrintInvoiceRequest")
        response = self._post("print/orders", json=invoice)
        return EntityResponse.model_validate(response)



from __future__ import annotations

from pydantic import Field

from ..models.intakes import Intakes


class IntakeRequest(Intakes):
    lunch_time_from: str | None = Field(default=None, description="Время начала обеда")  # type: ignore
    lunch_time_to: str | None = Field(default=None, description="Время окончания обеда")  # type: ignore



from .. import constants


class CdekException(Exception):
    """Базовое исключение для CDEK"""

    @staticmethod
    def get_translation(code: str, message: str) -> str:
        """Получить перевод ошибки"""
        if code in constants.ERRORS:
            return f"{constants.ERRORS[code]}. {message}"
        return message



from .barcode import BarcodeApp
from .invoice import InvoiceApp
from .requests import PrintBarcodeRequest, PrintInvoiceRequest
from .responses import (
    PrintBarcodeEntity,
    PrintBarcodeResponse,
    PrintStatus,
    WaybillEntityResponse,
    WaybillResponse,
)

__all__ = [
    "BarcodeApp",
    "InvoiceApp",
    "PrintBarcodeRequest",
    "PrintInvoiceRequest",
    "PrintBarcodeResponse",
    "WaybillEntityResponse",
    "PrintBarcodeEntity",
    "PrintStatus",
    "WaybillResponse",
]



from __future__ import annotations

from datetime import datetime

from pydantic import BaseModel, Field, field_serializer

from ..models.contact import ContragentType
from ..models.location import Coordinates, Zip
from ..models.package import CalcPackage


class CalculatorLocation(Zip, Coordinates):
    code: int | None = Field(default=None, description="Код населенного пункта СДЭК")
    country_code: str | None = Field(
        default=None,
        max_length=2,
        description="Код страны в формате ISO_3166-1_alpha-2",
    )
    city: str | None = Field(default=None, description="Название населенного пункта")
    address: str | None = Field(default=None, description="Адрес населенного пункта")
    contragent_type: ContragentType | None = Field(
        default=None, description="Тип контрагента"
    )


class TariffListRequest(BaseModel):
    date: datetime | None = Field(
        None,
        description="Дата и время планируемой передачи заказа",
    )
    type: int = Field(
        1,
        description="Тип заказа. 1 — интернет-магазин, 2 — доставка. По умолчанию — 1.",
    )
    additional_order_types: list[int] | None = Field(
        None,
        description="Дополнительные типы заказа",
    )
    currency: int | None = Field(
        None,
        description=("Валюта расчетаПо умолчанию — валюта договора."),
    )
    lang: str = Field(
        "rus",
        max_length=3,
        description="Язык вывода информации о тарифах. "
        "Возможные значения: rus, eng, zho",
    )
    from_location: CalculatorLocation | None = Field(
        None, description="Отправляющий адрес"
    )
    to_location: CalculatorLocation | None = Field(None, description="Получающий адрес")
    packages: list[CalcPackage] | None = Field(None, description="Список упаковок")

    def set_city_codes(self, from_location: int, to_location: int) -> TariffListRequest:
        """Задать коды населённых пунктов отправителя и получателя"""
        self.from_location = CalculatorLocation.model_validate({"code": from_location})
        self.to_location = CalculatorLocation.model_validate({"code": to_location})
        return self

    def set_package_weight(self, weight: int) -> TariffListRequest:
        """Определить вес единственной упаковки в граммах"""
        self.packages = [CalcPackage.model_validate({"weight": weight})]
        return self

    @field_serializer("date")
    def serialize_date(self, date: datetime) -> str:
        """Сериализовать дату в формат, поддерживаемый API"""
        return date.strftime("%Y-%m-%dT%H:%M:%S")


class CalcAdditionalService(BaseModel):
    code: str | None = Field(None, description="Код дополнительной услуги")
    parameter: str | None = Field(None, description="Параметр дополнительной услуги")


class TariffCodeRequest(TariffListRequest):
    tariff_code: int = Field(..., description="Код тарифа")
    services: list[CalcAdditionalService] | None = Field(
        None, description="Список дополнительных услуг"
    )



from enum import IntEnum

from cdek.apps.app import App
from cdek.exceptions import CdekRequestException


class Currency(IntEnum):
    """Enum для кодов валют, используемых в методах расчета стоимости CDEK."""

    RUB = 1  # Российский рубль
    KZT = 2  # Тенге
    USD = 3  # Доллар США
    EUR = 4  # Евро
    GBP = 5  # Фунт Стерлингов
    CNY = 6  # Китайский юань
    BYR = 7  # Белорусский рубль
    UAH = 8  # Гривна
    KGS = 9  # Киргизский сом
    AMD = 10  # Армянский драм
    TRY = 11  # Турецкая лира
    THB = 12  # Тайский бат
    KRW = 13  # Вон
    AED = 14  # Дирхам
    UZS = 15  # Узбекский сум
    MNT = 16  # Тугрик
    PLN = 17  # Злотый
    AZN = 18  # Манат
    GEL = 19  # Лари
    JPY = 55  # Йена
    VND = 704  # Вьетнамский донг


class CurrencyApp(App):
    def get(self, currency_code: str) -> int:
        """Получить числовой код валюты по её символьному обозначению"""
        try:
            return Currency[currency_code].value
        except ValueError as e:
            raise CdekRequestException(
                f"Неизвестный код валюты: {currency_code}"
            ) from e

    def all(self) -> list[int]:
        """Получить список всех поддерживаемых кодов валют"""
        return [currency.value for currency in Currency]



from __future__ import annotations

from typing import Any

from pydantic import BaseModel, Field

from .item import Item


class CalcPackage(BaseModel):
    weight: int | None = Field(None, description="Вес упаковки в граммах")
    length: int | None = Field(None, description="Длина упаковки в сантиметрах")
    width: int | None = Field(None, description="Ширина упаковки в сантиметрах")
    height: int | None = Field(None, description="Высота упаковки в сантиметрах")


class Package(CalcPackage):
    number: str = Field(..., description="Номер упаковки")
    weight: int = Field(..., description="Вес упаковки в граммах")
    comment: str | None = Field(None, description="Комментарий к упаковке")
    items: list[Item] | None = Field(None, description="Позиции товаров в упаковке")
    package_id: str | None = Field(
        None, description="Уникальный номер упаковки в ИС СДЭК"
    )

    def add_item(self, **kwargs: Any) -> Item:
        """Добавить товар в упаковку и вернуть созданный объект"""
        item = Item(**kwargs)
        if self.items is None:
            self.items = []
        self.items.append(item)
        return item



from __future__ import annotations

from datetime import date as Date
from typing import Any
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer

from .contact import Contact
from .location import Address, City, Coordinates, Country, Region, Zip


class IntakeLocation(City, Country, Region, Coordinates, Zip, Address):
    address: str = Field(..., description="Строка адреса")
    region: str | None = Field(None, description="Название региона")  # type: ignore
    city: str | None = Field(None, description="Название населенного пункта")  # type: ignore
    city_uuid: str | None = Field(
        None, description="Уникальный идентификатор населенного пункта"
    )  # type: ignore


class Intakes(BaseModel):
    cdek_number: str | None = Field(None, description="Номер заказа СДЭК")
    order_uuid: UUID | None = Field(None, description="Идентификатор заказа")
    intake_date: Date = Field(..., description="Дата заявки на вызов курьера")
    intake_time_from: str = Field(
        ..., description="Время начала заявки на вызов курьера"
    )
    intake_time_to: str = Field(
        ..., description="Время окончания заявки на вызов курьера"
    )
    lunch_time_from: str = Field(..., description="Время начала обеда")
    lunch_time_to: str = Field(..., description="Время окончания обеда")
    name: str | None = Field(None, description="Имя заказчика")
    weight: int | None = Field(None, description="Вес заявки")
    length: int | None = Field(None, description="Длина заявки")
    width: int | None = Field(None, description="Ширина заявки")
    height: int | None = Field(None, description="Высота заявки")
    comment: str | None = Field(None, description="Комментарий к заявке")
    courier_power_of_attorney: bool | None = Field(
        None, description="Нужно ли вызывать курьера"
    )
    courier_identity_card: bool | None = Field(
        None, description="Нужно ли вызывать курьера"
    )
    sender: Contact | None = Field(None, description="Контакт заказчика")
    from_location: IntakeLocation | None = Field(None, description="Населённый пункт")
    need_call: bool = Field(False, description="Необходим прозвон получателя")

    @field_serializer("intake_date")
    def serialize_intake_date(self, intake_date: Date) -> str:
        """Представить дату забора в формате YYYY-MM-DD"""
        return intake_date.strftime("%Y-%m-%d")

    def from_location_init(self, **kwargs: Any) -> IntakeLocation:
        """Создать объект адреса забора по переданным параметрам"""
        return IntakeLocation(**kwargs)

    def sender_init(self, **kwargs: Any) -> Contact:
        """Создать объект контакта заказчика"""
        return Contact(**kwargs)

    def set_sender(self, sender: Contact) -> Intakes:
        """Установить контактные данные заказчика"""
        self.sender = sender
        return self

    def set_from_location(self, location: IntakeLocation) -> Intakes:
        """Указать место забора"""
        self.from_location = location
        return self



from __future__ import annotations

from enum import Enum

from pydantic import BaseModel, Field


class PrintType(str, Enum):
    """Типы печатной формы"""

    tpl_china = "tpl_china"
    tpl_armenia = "tpl_armenia"
    tpl_russia = "tpl_russia"
    tpl_english = "tpl_english"
    tpl_italian = "tpl_italian"
    tpl_korean = "tpl_korean"
    tpl_latvian = "tpl_latvian"
    tpl_lithuanian = "tpl_lithuanian"
    tpl_german = "tpl_german"
    tpl_turkish = "tpl_turkish"
    tpl_czech = "tpl_czech"
    tpl_thailand = "tpl_thailand"
    tpl_invoice = "tpl_invoice"


class PrintOrder(BaseModel):
    order_uuid: str | None = Field(None, description="Идентификатор заказа в ИС СДЭК")
    cdek_number: int | None = Field(None, description="Номер заказа СДЭК")


class PrintForm(BaseModel):
    orders: list[PrintOrder] = Field(..., description="Список заказов")
    copy_count: int | None = Field(None, description="Количество копий")



from __future__ import annotations

from datetime import date as Date

from pydantic import BaseModel, Field, field_serializer


class WorkTime(BaseModel):
    day: int = Field(..., description="Порядковый номер дня начиная с единицы")
    time: str = Field(..., max_length=255, description="Период работы в эти дни")


class WorkTimeException(BaseModel):
    date_start: Date = Field(..., description="Дата начала исключения в работе офиса")
    date_end: Date = Field(..., description="Дата окончания исключения в работе офиса")
    time_start: str | None = Field(
        None, description="Время начала работы в указанную дату"
    )
    time_end: str | None = Field(
        None, description="Время окончания работы в указанную дату"
    )
    is_working: bool = Field(
        ..., description="Признак рабочего/нерабочего дня в указанную дату"
    )

    @field_serializer("date_start")
    def serialize_date_start(self, date_start: Date) -> str:
        """Представить дату начала исключения в формате ISO"""
        return date_start.strftime("%Y-%m-%d")

    @field_serializer("date_end")
    def serialize_date_end(self, date_end: Date) -> str:
        """Представить дату окончания исключения в формате ISO"""
        return date_end.strftime("%Y-%m-%d")



from ..app import App
from .requests import TariffCodeRequest, TariffListRequest
from .responses import TariffAvailableResponse, TariffListResponse, TariffResponse


class TariffApp(App):
    """Класс для работы с тарифами"""

    def calc_list(self, tariff: "TariffListRequest") -> "TariffListResponse":
        """Расчёт стоимости и сроков доставки по всем доступным тарифам

        Args:
            tariff: объект Tariff с параметрами запроса
            (type, from_location, to_location, packages)

        Returns:
            Список объектов TariffListResponse с информацией о доступных тарифах

        Raises:
            ValueError: если tariff не является объектом TariffListRequest
        """
        if not isinstance(tariff, TariffListRequest):
            raise ValueError("tariff must be a TariffListRequest")
        response = self._post("calculator/tarifflist", json=tariff)
        return TariffListResponse.model_validate(response)

    def calc(self, tariff: "TariffCodeRequest") -> "TariffResponse":
        """
        Расчёт стоимости и сроков доставки по коду тарифа

        Args:
            tariff: объект TariffCodeRequest с параметрами запроса
            (tariff_code, from_location, to_location, packages)

        Returns:
            TariffResponse: объект с информацией о расчёте стоимости и сроков доставки

        Raises:
            ValueError: если tariff не является объектом TariffCodeRequest
        """
        if not isinstance(tariff, TariffCodeRequest):
            raise ValueError("tariff must be a TariffCodeRequest")
        response = self._post("calculator/tariff", json=tariff)
        return TariffResponse.model_validate(response)

    def all(self) -> "TariffAvailableResponse":
        """
        Список доступных тарифов
        Метод позволяет получить список всех доступных и актуальных тарифов по договору

        Returns:
            Список объектов TariffAvailableResponse с информацией о доступных тарифах
        """
        response = self._get("calculator/alltariffs")
        return TariffAvailableResponse.model_validate(response)



from __future__ import annotations

from datetime import datetime

from pydantic import BaseModel, Field, field_serializer


class Status(BaseModel):
    code: str = Field(..., description="Код статуса")
    name: str = Field(..., description="Название статуса")
    date_time: datetime = Field(..., description="Дата и время статуса")

    @field_serializer("date_time")
    def serialize_date_time(self, date_time: datetime) -> str:
        """Преобразовать дату статуса в строку формата ISO"""
        return date_time.strftime("%Y-%m-%dT%H:%M:%S")



from ..app import App
from ..models.entity_response import EntityResponse
from .requests import OrderRequest, OrderUpdateRequest
from .responses import OrderEntityResponse


class OrderApp(App):
    """Класс для работы с заказами"""

    def create(self, order: "OrderRequest") -> "EntityResponse":
        """
        Создать заказ в системе CDEK

        Args:
            order: объект OrderRequest с параметрами запроса

        Returns:
            EntityResponse: объект с информацией о созданном заказе

        Raises:
            ValueError: если order не является объектом OrderRequest
        """
        if not isinstance(order, OrderRequest):
            raise ValueError("order must be a OrderRequest")
        response = self._post("orders", json=order)
        return EntityResponse.model_validate(response)

    def update(self, order: "OrderUpdateRequest") -> "EntityResponse":
        """
        Обновить сведения по ранее созданному заказу

        Args:
            order: объект OrderUpdateRequest с параметрами запроса

        Returns:
            EntityResponse: объект с информацией о обновленном заказе

        Raises:
            ValueError: если order не является объектом OrderUpdateRequest
        """
        if not isinstance(order, OrderUpdateRequest):
            raise ValueError("order must be a OrderUpdateRequest")
        response = self._patch("orders", json=order)
        return EntityResponse.model_validate(response)

    def delete(self, uuid: str) -> EntityResponse:
        """
        Удалить заказ по его UUID

        Args:
            uuid: идентификатор заказа

        Returns:
            EntityResponse: объект с информацией о удаленном заказе

        Raises:
            ValueError: если uuid не является строкой
        """
        if not isinstance(uuid, str):
            raise ValueError("uuid must be a str")
        response = self._delete(f"orders/{uuid}")
        return EntityResponse.model_validate(response)

    def refusal(self, uuid: str) -> EntityResponse:
        """Регастрация отказа

        Регистрации отказа по заказу и дальнейшего возврата
        данного заказа в интернет-магазин.
        После успешной регистрации отказа статус заказа переходит
        в "Не вручен" (код NOT_DELIVERED) с дополнительным
        статусом "Возврат, отказ от получения: Без объяснения" (код 11).
        Заказ может быть отменен в любом статусе*, пока не установлен статус
        "Вручен" или "Не вручен".
        * Не рекомендуется использовать метод для заказов,
        которые находятся в статусе "Создан" и не планируются к отгрузке
        на склады СДЭК. В случае применения метода отказа для заказов,
        находящихся в статусе "Создан", по ним будут начислены
        операции и заказы будут включены в Акт оказанных услуг.
        Для отмены заказа в статусе "Создан" воспользуйтесь методом
        "Удаление заказа".

        Args:
            uuid: идентификатор заказа

        Returns:
            EntityResponse: объект с информацией о регистрации отказа

        Raises:
            ValueError: если uuid не является строкой
        """
        if not isinstance(uuid, str):
            raise ValueError("uuid must be a str")
        response = self._post(f"orders/{uuid}/refusal")
        return EntityResponse.model_validate(response)

    def client_return(self, uuid: str, tariff_code: int) -> "EntityResponse":
        """Регистрация клиентского возврата

        Args:
            uuid: идентификатор заказа
            tariff_code: код тарифа

        Returns:
            EntityResponse: объект с информацией о регистрации клиентского возврата
        """
        response = self._post(
            f"orders/{uuid}/clientReturn",
            json={"tariff_code": tariff_code},
        )
        return EntityResponse.model_validate(response)

    def get_by_cdek_number(self, cdek_number: str) -> "OrderEntityResponse":
        """Полная информация о заказе по трек номеру

        Args:
            cdek_number: трек номер заказа

        Returns:
            OrderEntityResponse: объект с информацией о заказе
        """
        response = self._get("orders", params={"cdek_number": cdek_number})
        return OrderEntityResponse.model_validate(response)

    def get_by_im_number(self, im_number: str) -> OrderEntityResponse:
        """Полная информация о заказе по ID заказа в магазине

        Args:
            im_number: ID заказа в магазине

        Returns:
            OrderEntityResponse: объект с информацией о заказе
        """
        response = self._get("orders", params={"im_number": im_number})
        return OrderEntityResponse.model_validate(response)

    def get_by_uuid(self, uuid: str) -> OrderEntityResponse:
        """Полная информация о заказе по uuid

        Args:
            uuid: идентификатор заказа

        Returns:
            OrderEntityResponse: объект с информацией о заказе
        """
        response = self._get(f"orders/{uuid}")
        return OrderEntityResponse.model_validate(response)



from __future__ import annotations

from typing import Any

from pydantic import BaseModel, Field

from ..models.error import Error
from ..models.warning import WarningModel


class DeliveryDateRange(BaseModel):
    min: str | None = Field(None, description="Минимальная дата доставки")
    max: str | None = Field(None, description="Максимальная дата доставки")


class TariffListItem(BaseModel):
    """Модель элемента списка тарифов."""

    tariff_code: int = Field(
        ..., description="Код тарифа. Обязателен для расчета по коду тарифа"
    )
    tariff_name: str = Field(
        ..., max_length=255, description="Название тарифа на языке вывода"
    )
    tariff_description: str | None = Field(
        None, max_length=255, description="Описание тарифа на языке вывода"
    )
    delivery_mode: int = Field(..., description="Режим тарифа (справочник СДЭК)")
    delivery_sum: float = Field(..., description="Стоимость доставки")
    period_min: int = Field(
        ..., description="Минимальное время доставки (в рабочих днях)"
    )
    period_max: int = Field(
        ..., description="Максимальное время доставки (в рабочих днях)"
    )
    calendar_min: int | None = Field(
        None, description="Минимальное время доставки (в календарных днях)"
    )
    calendar_max: int | None = Field(
        None, description="Максимальное время доставки (в календарных днях)"
    )
    delivery_date_range: DeliveryDateRange | None = Field(
        None, description="Прогнозируемый диапазон дат доставки"
    )


class TariffListResponse(BaseModel):
    """Модель ответа со списком тарифов."""

    tariff_codes: list[TariffListItem] | None = Field(default_factory=list)
    errors: list[Error] | None = Field(None, description="Список ошибок")
    warnings: list[WarningModel] | None = Field(
        None, description="Список предупреждений"
    )

    def get_codes(self) -> list[TariffListItem]:
        """Вернуть список тарифов, гарантируя, что результат не None"""
        return self.tariff_codes or []


class Services(BaseModel):
    """Модель ответа для дополнительных услуг."""

    code: str = Field(..., description="Тип дополнительной услуги")
    sum: float = Field(..., description="Стоимость услуги")
    total_sum: float = Field(..., description="Стоимость услуги с НДС и скидкой")
    discount_percent: float = Field(..., description="Процент скидки")
    discount_sum: float = Field(..., description="Сумма скидки")
    vat_rate: float = Field(..., description="Ставка НДС")
    vat_sum: float = Field(..., description="Сумма НДС")


class TariffResponse(BaseModel):
    delivery_sum: float = Field(..., description="Стоимость доставки")
    period_min: int = Field(
        ..., description="Минимальное время доставки (в рабочих днях)"
    )
    period_max: int = Field(
        ..., description="Максимальное время доставки (в рабочих днях)"
    )
    calendar_min: int | None = Field(
        None, description="Минимальное время доставки (в календарных днях)"
    )
    calendar_max: int | None = Field(
        None, description="Максимальное время доставки (в календарных днях)"
    )
    weight_calc: int = Field(..., description="Расчетный вес (в граммах)")
    services: list[Services] | None = Field(
        None,
        description="Дополнительные услуги "
        "(возвращается, если в запросе были переданы доп. услуги)",
    )
    total_sum: float = Field(
        ..., description="Стоимость доставки с учетом дополнительных услуг"
    )
    currency: str = Field(
        ..., description="Валюта, в которой рассчитана стоимость доставки (код СДЭК)"
    )
    errors: list[Error] | None = Field(None, description="Список ошибок")
    warnings: list[WarningModel] | None = Field(
        None, description="Список предупреждений"
    )
    delivery_date_range: DeliveryDateRange | None = Field(
        None, description="Прогнозируемый диапазон дат доставки"
    )

    def get_calendar_min(self) -> int | None:
        """Получить минимальный период доставки."""
        return self.calendar_min

    def get_calendar_max(self) -> int | None:
        """Получить максимальный период доставки."""
        return self.calendar_max


class DeliveryMode(BaseModel):
    """Модель для режима доставки."""

    delivery_mode: str | None = None
    delivery_mode_name: str | None = None
    tariff_code: int | None = None


class AvailableTariff(BaseModel):
    tariff_name: str | None = Field(None, description="Имя сервиса")
    weight_min: float | None = Field(None, description="Минимальный вес отправления")
    weight_max: float | None = Field(None, description="Максимальный вес отправления")
    weight_calc_max: float | None = Field(
        None, description="Максимальный расчётный вес"
    )
    length_min: int | None = Field(None, description="Минимальная длина упаковки")
    length_max: int | None = Field(None, description="Максимальная длина упаковки")
    width_min: int | None = Field(None, description="Минимальная ширина упаковки")
    width_max: int | None = Field(None, description="Максимальная ширина упаковки")
    height_min: int | None = Field(None, description="Минимальная высота упаковки")
    height_max: int | None = Field(None, description="Максимальная высота упаковки")
    order_types: list[str] | None = Field(
        None,
        description="Список доступных типов заказов для тарифа "
        "(пустой список означает отсутствие ограничений)",
    )
    payer_contragent_type: list[str] | None = Field(
        None,
        description=(
            "Список типов контрагентов-плательщиков: "
            "LEGAL_ENTITY — юридическое лицо, INDIVIDUAL — физическое лицо. "
            "Пустой список означает отсутствие ограничений."
        ),
    )
    sender_contragent_type: list[str] | None = Field(
        None,
        description=(
            "Список типов контрагентов-отправителей: "
            "LEGAL_ENTITY — юридическое лицо, INDIVIDUAL — физическое лицо. "
            "Пустой список означает отсутствие ограничений."
        ),
    )
    recipient_contragent_type: list[str] | None = Field(
        None,
        description=(
            "Список типов контрагентов-получателей: LEGAL_ENTITY — юридическое лицо, "
            "INDIVIDUAL — физическое лицо. "
            "Пустой список означает отсутствие ограничений."
        ),
    )
    delivery_modes: list[DeliveryMode] | None = Field(
        None, description="Режимы доставки"
    )
    additional_order_types_param: dict[str, Any] | None = Field(
        None, description="Доп. типы заказа, применимые к тарифу"
    )


class TariffAvailableResponse(BaseModel):
    """Модель ответа о доступных тарифах."""

    tariff_codes: list[AvailableTariff] | None = Field(
        None, description="Список доступных тарифов"
    )

    def get_codes(self) -> list[AvailableTariff]:
        """Вернуть список доступных тарифов без значений None"""
        return self.tariff_codes or []



from pydantic import BaseModel, Field

from .enums import WebhookType


class WebhookRequest(BaseModel):
    """Модель запроса на добавление подписки на webhook"""

    type: WebhookType = Field(..., description="Тип вебхука")
    url: str = Field(..., description="URL, на который отправляется событие")



from __future__ import annotations

from datetime import date as Date
from datetime import datetime
from typing import Any
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer

from ..models import Address, City, Coordinates, Country, Region, Zip
from ..models.order import OrderInfo


class ScheduleLocation(City, Country, Region, Coordinates, Address, Zip):
    code: int | None = Field(default=None)  # type: ignore
    region: str | None = Field(default=None)  # type: ignore
    region_code: int | None = Field(default=None)
    city: str | None = Field(default=None)  # type: ignore
    city_uuid: str | None = Field(default=None)  # type: ignore


class DeliveryLocation(Zip, Coordinates, Country, Region):
    code: int | None = None
    fias_guid: UUID | None = None
    city: str | None = None
    address: str


class RegisterDeliveryRequest(OrderInfo):
    date: Date = Field(..., description="Дата доставки, согласованная с получателем")
    time_from: str = Field(
        ..., description='Время доставки "С", согласованное с получателем'
    )
    time_to: str = Field(
        ..., description='Время доставки "По", согласованное с получателем'
    )
    comment: str | None = Field(
        None, max_length=255, description="Комментарий к договоренности о доставке"
    )
    delivery_point: str | None = Field(
        None, max_length=255, description="Буквенно-цифровой код ПВЗ СДЭК"
    )
    to_location: ScheduleLocation | None = Field(None, description="Населённый пункт")

    def to_location_init(self, **kwargs: Any) -> ScheduleLocation:
        """Создать объект локации для договорённости о доставке"""
        return ScheduleLocation(**kwargs)

    def set_to_location(self, location: ScheduleLocation) -> RegisterDeliveryRequest:
        """Установить локацию доставки"""
        self.to_location = location
        return self

    @field_serializer("date")
    def serialize_date(self, date: Date) -> str:
        """Вернуть дату доставки в формате YYYY-MM-DD"""
        return date.strftime("%Y-%m-%d")


class DeliveryIntervalRequest(BaseModel):
    date_time: datetime = Field(..., description="Дата и время заявки на вызов курьера")
    from_location: DeliveryLocation | None = Field(
        None, description="Адрес отправления"
    )
    shipment_point: str | None = Field(None, description="Код ПВЗ СДЭК")
    to_location: DeliveryLocation = Field(..., description="Адрес доставки")
    tariff_code: int = Field(..., description="Код тарифа")
    additional_order_types: list[int] | None = Field(
        None, description="Дополнительные типы заказа"
    )

    @field_serializer("date_time")
    def serialize_date_time(self, date_time: datetime) -> str:
        """Вернуть дату запроса интервалов в формате ISO 8601"""
        return date_time.strftime("%Y-%m-%dT%H:%M:%S")



from .filters import IntakeDateFilter, IntakeFilter, IntakeLocation, IntakeStatus
from .intake import IntakeApp
from .requests import IntakeRequest
from .responses import (
    IntakeDateResponse,
    IntakeEntityResponse,
    IntakePackage,
    IntakesEntity,
    IntakesResponse,
)

__all__ = [
    "IntakeApp",
    "IntakeFilter",
    "IntakeDateFilter",
    "IntakeRequest",
    "IntakeDateResponse",
    "IntakeEntityResponse",
    "IntakeStatus",
    "IntakeLocation",
    "IntakesResponse",
    "IntakePackage",
    "IntakesEntity",
]



from __future__ import annotations

from pydantic import BaseModel, Field


class Seller(BaseModel):
    """Модель для реквизитов истинного продавца"""

    name: str | None = Field(None, description="Наименование истинного продавца")
    inn: str | None = Field(None, description="ИНН истинного продавца")
    phone: str | None = Field(None, description="Телефон истинного продавца")
    ownership_form: int | None = Field(None, description="Код формы собственности")
    address: str | None = Field(None, description="Адрес истинного продавца")



from __future__ import annotations

from datetime import date as Date
from enum import Enum

from pydantic import BaseModel, Field, field_serializer

from .phone import Phone


class ContragentType(str, Enum):
    LEGAL_ENTITY = "LEGAL_ENTITY"
    INDIVIDUAL = "INDIVIDUAL"


class Tin(BaseModel):
    tin: str | None = Field(None, max_length=255, description="ИНН")


class Passport(BaseModel):
    passport_series: str | None = Field(
        None, max_length=255, description="Серия паспорта"
    )
    passport_number: str | None = Field(
        None, max_length=255, description="Номер паспорта"
    )
    passport_date_of_issue: Date | None = Field(
        None, description="Дата выдачи паспорта"
    )
    passport_organization: str | None = Field(
        None, max_length=255, description="Организация выдавшая паспорт"
    )
    passport_date_of_birth: Date | None = Field(None, description="Дата рождения")

    @field_serializer("passport_date_of_issue")
    def serialize_passport_date_of_issue(self, passport_date_of_issue: Date) -> str:
        """Вернуть дату выдачи паспорта в формате YYYY-MM-DD"""
        return passport_date_of_issue.strftime("%Y-%m-%d")

    @field_serializer("passport_date_of_birth")
    def serialize_passport_date_of_birth(self, passport_date_of_birth: Date) -> str:
        """Вернуть дату рождения в формате YYYY-MM-DD"""
        return passport_date_of_birth.strftime("%Y-%m-%d")


class Contact(Passport, Tin):
    company: str | None = Field(None, description="Название компании")
    name: str | None = Field(None, description="Имя заказчика")
    contragent_type: ContragentType | None = Field(None, description="Тип контрагента")
    email: str | None = Field(None, description="Email")
    phones: list[Phone] = Field(default_factory=list, description="Телефоны")

    def add_phone(self, number: str, additional: str | None = None) -> Contact:
        """Добавить новый номер телефона в контакт"""
        phone = Phone(number=number, additional=additional)
        self.phones.append(phone)
        return self



"""Модель для работы с телефоном."""

from __future__ import annotations

from pydantic import BaseModel, Field


class Phone(BaseModel):
    """Модель для телефонного номера."""

    number: str = Field(..., max_length=24, description="Номер телефона")
    additional: str | None = Field(
        None, max_length=255, description="Дополнительная информация (добавочный номер)"
    )



# Ошибка авторизации
AUTH_FAIL = "Аутентификация не удалась, пожалуйста, проверьте переданные логин и пароль"

DELIVERY_METHODS = {
    "1": ("door", "door"),
    "2": ("door", "office"),
    "3": ("office", "door"),
    "4": ("office", "office"),
    "5": ("terminal", "terminal"),
    "6": ("door", "postbox"),
    "7": ("office", "postbox"),
    "8": ("postbox", "door"),
    "9": ("postbox", "office"),
    "10": ("postbox", "postbox"),
}
# Коды услуг
SERVICE_CODES = {
    "INSURANCE": "Страхование",
    "TAKE_SENDER": "Забор в городе отправителе",
    "DELIV_RECEIVER": "Доставка в городе получателе",
    "TRYING_ON": "Примерка на дому",
    "PART_DELIV": "Частичная доставка",
    "INSPECTION_CARGO": "Осмотр вложения",
    "REVERSE": "Реверс",
    "DANGER_CARGO": "Опасный груз",
    "PACKAGE_1": "Упаковка 1",
    "PACKAGE_2": "Упаковка 2",
    "WAIT_FOR_RECEIVER": "Ожидание более 15 мин. у получателя",
    "WAIT_FOR_SENDER": "Ожидание более 15 мин. у отправителя",
    "REPEATED_DELIVERY": "Повторная поездка",
    "SMS": "Смс уведомление",
    "GET_UP_FLOOR_BY_HAND": "Подъем на этаж ручной",
    "GET_UP_FLOOR_BY_ELEVATOR": "Подъем на этаж лифтом",
    "CALL": "Прозвон",
    "THERMAL_MODE": "Тепловой режим",
    "COURIER_PACKAGE_A2": "Пакет курьерский А2",
    "SECURE_PACKAGE_A2": "Сейф пакет А2",
    "SECURE_PACKAGE_A3": "Сейф пакет А3",
    "SECURE_PACKAGE_A4": "Сейф пакет А4",
    "SECURE_PACKAGE_A5": "Сейф пакет А5",
    "NOTIFY_ORDER_CREATED": "Уведомление о создании заказа в СДЭК",
    "NOTIFY_ORDER_DELIVERY": "Уведомление о приеме заказа на доставку",
    "CARTON_BOX_XS": "Коробка XS (0,5 кг 17х12х9 см)",
    "CARTON_BOX_S": "Коробка S (2 кг 21х20х11 см)",
    "CARTON_BOX_M": "Коробка M (5 кг 33х25х15 см)",
    "CARTON_BOX_L": "Коробка L (12 кг 34х33х26 см)",
    "CARTON_BOX_500GR": "Коробка (0,5 кг 17х12х10 см)",
    "CARTON_BOX_1KG": "Коробка (1 кг 24х17х10 см)",
    "CARTON_BOX_2KG": "Коробка (2 кг 34х24х10 см)",
    "CARTON_BOX_3KG": "Коробка (3 кг 24х24х21 см)",
    "CARTON_BOX_5KG": "Коробка (5 кг 40х24х21 см)",
    "CARTON_BOX_10KG": "Коробка (10 кг 40х35х28 см)",
    "CARTON_BOX_15KG": "Коробка (15 кг 60х35х29 см)",
    "CARTON_BOX_20KG": "Коробка (20 кг 47х40х43 см)",
    "CARTON_BOX_30KG": "Коробка (30 кг 69х39х42 см)",
    "BUBBLE_WRAP": "Воздушно-пузырчатая пленка",
    "WASTE_PAPER": "Макулатурная бумага",
    "CARTON_FILLER": 'Прессованный картон "филлер" (55х14х2,3 см)',
    "BAN_ATTACHMENT_INSPECTION": "Запрет осмотра вложения",
}

# Коды ошибок
ERRORS = {
    "v2_internal_error": "Запрос выполнился с системной ошибкой",
    "v2_similar_request_still_processed": "Предыдущий запрос такого же типа над этой же сущностью еще не выполнился",  # noqa: E501
    "v2_bad_request": "Передан некорректный запрос",
    "v2_invalid_format": "Передано некорректное значение",
    "v2_field_is_empty": "Не передано обязательное поле",
    "v2_parameters_empty": "Все параметры запроса пустые или не переданы",
    "v2_invalid_value_type": "Передан некорректный тип данных",
    "v2_entity_not_found": "Сущность (заказ, заявка и т.д.) с указанным идентификатором не существует либо удалена",  # noqa: E501
    "v2_entity_forbidden": "Сущность (заказ, заявка и т.д.) с указанным идентификатором существует, но принадлежит другому клиенту",  # noqa: E501
    "v2_entity_invalid": "Сущность (заказ, заявка и т.д.) с указанным идентификатором существует, но некорректна",  # noqa: E501
    "v2_order_not_found": "Заказ с указанным номером не существует либо удален",
    "v2_order_forbidden": "Заказ с указанным номером существует, но принадлежит другому клиенту",  # noqa: E501
    "v2_order_number_empty": "Не переданы номер и идентификатор заказа",
    "v2_shipment_address_multivalued": "Одновременно переданы ПВЗ отправителя и адрес отправителя. Необходимо указать 1 параметр",  # noqa: E501
    "v2_delivery_address_multivalued": "Одновременно переданы ПВЗ получателя и адрес получателя. Необходимо указать 1 параметр.",  # noqa: E501
    "v2_sender_location_not_recognized": "Не удалось определить город отправителя",
    "v2_recipient_location_not_recognized": "Не удалось определить город получателя",
    "v2_number_items_is_more_126": "Количество позиций товаров в заказе свыше 126",
    "orders_number_is_empty": "Все заказы с указанными номерами и идентификаторами некорректны",  # noqa: E501
    "shipment_location_is_not_recognized": "Передан некорректный код ПВЗ отправителя",
    "delivery_location_is_not_recognized": "Передан некорректный код ПВЗ получателя",
    "v2_entity_not_found_im_number": "Сущность с указанным идентификатором не существует либо удалена",  # noqa: E501
    "v2_order_location_from_and_shipment_point_empty": "Адрес отправителя и ПВЗ отправителя пустые",  # noqa: E501
    "v2_order_location_to_and_delivery_point_empty": "Адрес получателя и ПВЗ получателя пустые",  # noqa: E501
    "v2_number_items_in_package_is_more_126": "Позиций в товаре более 126",
    "v2_package_id_is_empty": "Не передан идентификатор уже существующей упаковки заказа",  # noqa: E501
    "v2_required_param_empty": "Не заполнено обязательное поле",
    "v2_city_can_not_be_changed": "Передан новый город в адресе доставки (для регистрации договоренности)",  # noqa: E501
    "v2_impossible_register_date": 'Истинный режим переданного заказа - "До постамата"',
    "v2_intake_exists_by_order": "К переданному заказу уже привязана заявка",
    "v2_intake_exists_by_date_address": "На переданную дату и в переданный адрес уже есть заявка",  # noqa: E501
    "v2_webhook_type_incorrect": "Передан некорректный тип события",
    "v2_entity_empty": "Все заказы с указанными номерами и идентификаторами некорректны",  # noqa: E501
    "v2_wrong_interval": "Передан некорректный промежуток времени",
    "v2_entity_expired": "Истек срок хранения квитанции с указанным идентификатором",
    "v2_invoice_entity_empty": "Квитанция с указанным идентификатором не сформировалась, так как все заказы некорректны",  # noqa: E501
    "v2_print_form_expired": "Истек срок хранения печатной формы с указанным идентификатором",  # noqa: E501
    "v2_print_form_empty": "Печатная форма с указанным идентификатором не сформировалась, так как все заказы некорректны",  # noqa: E501
    "v2_unable_add_service": "Не удалось добавить услугу к заказу.",
}



from __future__ import annotations

from datetime import date as Date
from datetime import datetime
from enum import Enum
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer, field_validator


class RelatedEntityType(str, Enum):
    """Типы связанных сущностей"""

    RETURN_ORDER = "RETURN_ORDER"
    DIRECT_ORDER = "DIRECT_ORDER"
    CLIENT_RETURN_ORDER = "CLIENT_RETURN_ORDER"
    CLIENT_DIRECT_ORDER = "CLIENT_DIRECT_ORDER"
    WAYBILL = "WAYBILL"
    BARCODE = "BARCODE"
    REVERSE_ORDER = "REVERSE_ORDER"
    DELIVERY = "DELIVERY"


class RelatedEntity(BaseModel):
    """Связанные сущности"""

    uuid: UUID | None = Field(None, description="Идентификатор сущности в ИС СДЭК")
    type: RelatedEntityType = Field(..., description="Тип связанной сущности")
    url: str | None = Field(
        None, max_length=255, description="Ссылка на скачивание печатной формы"
    )
    create_time: datetime | None = Field(
        None, description="Время создания связанной сущности"
    )
    cdek_number: str | None = Field(
        None, max_length=255, description="Номер заказа СДЭК"
    )
    date: Date | None = Field(
        None, description="Дата доставки, согласованная с получателем"
    )
    time_from: str | None = Field(None, description="Время начала ожидания курьера")
    time_to: str | None = Field(None, description="Время окончания ожидания курьера")

    @field_validator("type", mode="before")
    @classmethod
    def convert_type_to_upper(cls, v: str) -> str:
        """Преобразование типа в верхний регистр перед валидацией"""
        return isinstance(v, str) and v.upper() or v

    @field_serializer("create_time")
    def serialize_create_time(self, create_time: datetime) -> str:
        """Конвертировать время создания в строку ISO"""
        return create_time.strftime("%Y-%m-%dT%H:%M:%S")

    @field_serializer("date")
    def serialize_date(self, date: Date) -> str:
        """Конвертировать дату доставки в формат YYYY-MM-DD"""
        return date.strftime("%Y-%m-%d")



from __future__ import annotations

from ..app import App
from .filters import OfficeFilter
from .responses import OfficeResponse


class OfficeApp(App):
    """Класс для работы с ПВЗ СДЭК"""

    @property
    def headers(self) -> dict[str, str]:
        return {
            key: value
            for key, value in self._response_headers.items()
            if key.startswith("X-")
        }

    def _get_offices(
        self, filter_params: OfficeFilter | None = None
    ) -> list[OfficeResponse]:
        """Получение списка ПВЗ СДЭК

        Args:
            filter_params (OfficeFilter): фильтр для получения списка ПВЗ СДЭК

        Returns:
            list[OfficeResponse]: список объектов OfficeResponse с информацией о ПВЗ

        Raises:
            ValueError: если filter_params не является объектом OfficeFilter
        """
        if filter_params is not None and not isinstance(filter_params, OfficeFilter):
            raise ValueError("filter_params must be a OfficeFilter instance")
        response = self._get("deliverypoints", params=filter_params)
        return [OfficeResponse.model_validate(office) for office in response]

    def get(
        self, filter_params: OfficeFilter | None = None
    ) -> dict[str, list[OfficeResponse] | dict]:  # type: ignore
        """Получение списка ПВЗ СДЭК

        Args:
            filter_params (OfficeFilter): фильтр для получения списка ПВЗ СДЭК

        Returns:
            list[OfficeResponse]: список объектов OfficeResponse с информацией о ПВЗ
            dict: заголовки ответа
        """
        return {"result": self._get_offices(filter_params), "headers": self.headers}



from .agreement import AgreementApp
from .requests import (
    DeliveryIntervalRequest,
    DeliveryLocation,
    RegisterDeliveryRequest,
    ScheduleLocation,
)
from .responses import (
    AgreementInfoResponse,
    AvailableDeliveryInterval,
    AvailableDeliveryIntervalsInfo,
    AvailableDeliveryIntervalsResponse,
    ScheduleInfoEntity,
)

__all__ = [
    "AgreementApp",
    "DeliveryIntervalRequest",
    "RegisterDeliveryRequest",
    "AgreementInfoResponse",
    "AvailableDeliveryIntervalsResponse",
    "ScheduleInfoEntity",
    "AvailableDeliveryInterval",
    "AvailableDeliveryIntervalsInfo",
    "ScheduleLocation",
    "DeliveryLocation",
]



from typing import Any

from .. import constants
from .cdek_exception import CdekException


class CdekRequestException(CdekException):
    """Ошибка запроса к API CDEK"""

    def __init__(
        self, message: str, status_code: int | None = None, response: Any | None = None
    ):
        """Инициализировать исключение с сообщением и HTTP-статусом"""
        super().__init__(message)
        self.status_code = status_code
        self.response = response

    @staticmethod
    def get_translation(code: str, message: str) -> str:
        """Получить перевод ошибки"""
        if code in constants.ERRORS:
            return f"{constants.ERRORS[code]}. {message}"
        return message



from __future__ import annotations

from typing import Literal

from pydantic import Field

from ..models.print import PrintForm, PrintType


class PrintBarcodeRequest(PrintForm):
    format: Literal["A4", "A5", "A6", "A7"] = Field("A4", description="Формат печати")
    lang: Literal["RUS", "ENG"] = Field("RUS", description="Язык печати")


class PrintInvoiceRequest(PrintForm):
    type: PrintType | None = Field(None, description="Тип накладной")



from __future__ import annotations

from datetime import date as Date
from datetime import datetime

from pydantic import Field, field_serializer

from ..models.entity_response import EntityResponse
from ..models.order import (
    AccompanyingWaybill,
    Call,
    DeliveryDetail,
    DeliveryProblem,
    Order,
    OrderStatus,
)


class OrderResponse(Order):
    """Модель ответа о заказе."""

    uuid: str = Field(..., description="Идентификатор заказа в ИС СДЭК")
    type: int = Field(..., description="Тип заказа")
    is_return: bool | None = Field(None, description="Признак возврата заказа")
    is_reverse: bool | None = Field(None, description="Признак реверсного заказа")
    cdek_number: int | None = Field(None, description="Номер заказа в ИС СДЭК")
    accompanying_waybill: AccompanyingWaybill | None = Field(
        None, description="Информация для сопроводительной накладной"
    )
    keep_free_until: datetime | None = Field(
        None, description="Дата окончания бесплатного хранения"
    )
    statuses: list[OrderStatus] = Field(..., description="Статусы заказа")
    is_client_return: bool | None = Field(
        None, description="Признак клиентского возврата"
    )
    delivery_mode: str | None = Field(None, description="Режим доставки")
    planned_delivery_date: Date | None = Field(
        None, description="Планируемая дата доставки"
    )
    delivery_detail: DeliveryDetail | None = Field(
        None, description="Детальная информация о доставке"
    )
    transacted_payment: bool | None = Field(
        None, description="Признак проведенного платежа"
    )
    delivery_problem: DeliveryProblem | None = Field(
        None, description="Проблема с доставкой"
    )
    developer_key: str | None = Field(None, description="Ключ разработчика")
    calls: Call | None = Field(None, description="Информация о прозвонах")

    @field_serializer("planned_delivery_date")
    def serialize_planned_delivery_date(self, planned_delivery_date: Date) -> str:
        """Вернуть плановую дату доставки в формате YYYY-MM-DD"""
        return planned_delivery_date.strftime("%Y-%m-%d")

    @field_serializer("keep_free_until")
    def serialize_keep_free_until(self, keep_free_until: datetime) -> str:
        """Представить дату окончания бесплатного хранения в формате ISO 8601"""
        return keep_free_until.strftime("%Y-%m-%dT%H:%M:%S")


class OrderEntityResponse(EntityResponse):
    """Модель ответа о заказе."""

    entity: OrderResponse | None = Field(
        default=None, description="Информация о заказе в рамках протокола"
    )



from .payment import PaymentApp
from .responses import (
    PaymentInfoResponse,
    PaymentOrder,
    PaymentResponse,
    Registry,
    RegistryOrder,
)

__all__ = [
    "PaymentApp",
    "PaymentInfoResponse",
    "PaymentResponse",
    "PaymentOrder",
    "RegistryOrder",
    "Registry",
]



"""Setup script for cdek package."""

try:
    from setuptools import find_packages, setup
except (
    ModuleNotFoundError
) as exc:  # pragma: no cover - выполняется только при отсутствии зависимости
    raise RuntimeError(
        "Для сборки пакета требуется установить setuptools: pip install setuptools"
    ) from exc

# Читаем содержимое README
try:
    with open("README.md", encoding="utf-8") as fh:
        long_description = fh.read()
except FileNotFoundError:
    long_description = "Python SDK для работы с API СДЭК версии 2.0"

setup(
    name="cdek",
    version="1.0.1",
    description="Python SDK для работы с API СДЭК версии 2.0",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="CDEK Python SDK",
    author_email="geomer198@gmail.com",
    url="https://github.com/xxp-odoo-erp/cdek",
    packages=find_packages(),
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
        "Programming Language :: Python :: 3.12",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Libraries",
    ],
    python_requires=">=3.10",
    install_requires=[
        "pydantic>=2.6,<3.0",
        "requests>=2.28.0",
    ],
    include_package_data=True,
    zip_safe=False,
)



from datetime import date as Date

from ..app import App
from .responses import PaymentInfoResponse, PaymentResponse


class PaymentApp(App):
    """Класс для работы с переводами наложенного платежа"""

    def get(self, date: Date) -> PaymentInfoResponse:
        """
        Запрос на получение информации о переводе наложенного платежа

        Args:
            date: дата для получения информации о переводе наложенного платежа

        Returns:
            PaymentInfoResponse: объект с информацией о переводе наложенного платежа

        Raises:
            ValueError: если date не является объектом Date
        """
        if not isinstance(date, Date):
            raise ValueError("date must be a Date")
        formatted_date_string = date.strftime("%Y-%m-%d")
        response = self._get("payment", params={"date": formatted_date_string})
        return PaymentInfoResponse.model_validate(response)

    def get_registries(self, date: Date) -> PaymentResponse:
        """
        Получение информации о реестрах НП

        Args:
            date: дата для получения информации о реестрах НП

        Returns:
            PaymentResponse: объект с информацией о реестрах НП

        Raises:
            ValueError: если date не является объектом Date
        """
        if not isinstance(date, Date):
            raise ValueError("date must be a Date")
        formatted_date_string = date.strftime("%Y-%m-%d")
        response = self._get("registries", params={"date": formatted_date_string})
        return PaymentResponse.model_validate(response)



from enum import Enum


class WebhookType(str, Enum):
    """Типы вебхуков."""

    # событие по статусам
    ORDER_STATUS = "ORDER_STATUS"
    # готовность печатной формы
    ORDER_MODIFIED = "ORDER_MODIFIED"
    # получение информации о закрытии преалерта
    PRINT_FORM = "PRINT_FORM"
    #
    RECEIPT = "RECEIPT"
    DOWNLOAD_PHOTO = "DOWNLOAD_PHOTO"
    PREALERT_CLOSED = "PREALERT_CLOSED"
    ACCOMPANYING_WAYBILL = "ACCOMPANYING_WAYBILL"
    OFFICE_AVAILABILITY = "OFFICE_AVAILABILITY"
    DELIV_PROBLEM = "DELIV_PROBLEM"
    DELIV_AGREEMENT = "DELIV_AGREEMENT"



from __future__ import annotations

from datetime import date as Date
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer

from ..models.location import Coordinates, Region, Zip


class IntakeStatus(BaseModel):
    code: str = Field(..., description="Код статуса")
    add_status: str = Field(..., description="Дополнительный код статуса")


class IntakeFilter(BaseModel):
    """Фильтр для заявок на вызов курьера"""

    uuid: UUID = Field(..., description="Идентификатор заявки в ИС СДЭК")
    status: IntakeStatus = Field(..., description="Статус заявки")


class IntakeLocation(Region, Coordinates, Zip):
    code: int | None = Field(default=None, description="Код населенного пункта СДЭК")
    city: str | None = Field(default=None, description="Название населенного пункта")
    fias_guid: UUID | None = Field(
        None, description="Уникальный идентификатор ФИАС населенного пункта"
    )
    address: str | None = Field(default=None, description="Строка адреса")
    region: str | None = Field(default=None, description="Название региона")  # type: ignore
    city_uuid: str | None = Field(
        default=None, description="Уникальный идентификатор населенного пункта"
    )


class IntakeDateFilter(BaseModel):
    from_location: IntakeLocation = Field(..., description="Адрес отправления")
    date: Date = Field(
        ..., description="До какого числа включительно получить доступные дни"
    )

    @field_serializer("date")
    def serialize_date(self, date: Date) -> str:
        """Представить дату фильтра в формате YYYY-MM-DD"""
        return date.strftime("%Y-%m-%d")



from __future__ import annotations

from datetime import datetime as DateTime
from typing import Literal

from pydantic import BaseModel, Field, field_serializer

from ..models.error import Error
from ..models.payment_info import PaymentInfo
from ..models.warning import WarningModel


class CheckInfo(BaseModel):
    order_uuid: str = Field(..., description="Идентификатор заказа")
    cdek_number: str = Field(..., description="Номер заказа СДЭК")
    date: DateTime = Field(..., description="Дата создания чека в формате YYYY-MM-DD")
    document_number: str = Field(..., description="Номер документа")
    fiscal_sign: int = Field(..., description="Фискальный признак")
    type: Literal["CASH_RECEIPT_IN", "CASH_RECEIPT_REFUND"] = Field(
        ..., description="Тип чека"
    )
    payment_info: list[PaymentInfo] = Field(..., description="Информация о платеже")
    shift_no: int | None = Field(None, description="Номер смены")

    @field_serializer("date")
    def serialize_date(self, date: DateTime) -> str:
        """Вернуть дату формирования чека в формате YYYY-MM-DD"""
        return date.strftime("%Y-%m-%d")


class CheckResponse(BaseModel):
    check_info: list[CheckInfo] = Field(..., description="Информация о чеке")
    errors: list[Error] | None = Field(None, description="Список ошибок")
    warnings: list[WarningModel] | None = Field(
        None, description="Список предупреждений"
    )



from __future__ import annotations

from pydantic import BaseModel, Field


class Error(BaseModel):
    """Ошибки"""

    code: str | None = Field(None, description="Код ошибки")
    additional_code: str | None = Field(
        None, description="Дополнительный код ошибки для службы поддержки"
    )
    message: str | None = Field(None, description="Описание ошибки")



from __future__ import annotations

from typing import Literal

from pydantic import BaseModel, Field


class PaymentInfo(BaseModel):
    sum: float = Field(..., description="Сумма платежа")
    type: Literal["CASH", "CARD"] = Field(..., description="Тип платежа")



from __future__ import annotations

from ..app import App
from .requests import WebhookRequest
from .responses import (
    WebhookDeleteEntityResponse,
    WebhookResponse,
    WebhookUUIDEntityResponse,
    WebookSetEntityResponse,
)


class WebhookApp(App):
    """Класс для работы с webhook"""

    def all(self) -> list[WebhookResponse]:
        """Информация о слушателях webhook"""
        response = self._get("webhooks")
        return [WebhookResponse.model_validate(item) for item in response]

    def get(self, uuid: str) -> WebhookUUIDEntityResponse:
        """
        Информация о слушателе webhook

        Args:
            uuid: идентификатор слушателя webhook

        Returns:
            WebhookUUIDEntityResponse: объект с информацией о слушателе webhook
        """
        response = self._get(f"webhooks/{uuid}")
        return WebhookUUIDEntityResponse.model_validate(response)

    def delete(self, uuid: str) -> WebhookDeleteEntityResponse:
        """
        Удаление слушателя webhook

        Args:
            uuid: идентификатор слушателя webhook

        Returns:
            WebhookDeleteEntityResponse: объект с информацией о удаленном
                слушателе webhook
        """
        if not isinstance(uuid, str):
            raise ValueError("uuid must be a str")
        response = self._delete(f"webhooks/{uuid}")
        return WebhookDeleteEntityResponse.model_validate(response)

    def set(self, webhook: WebhookRequest) -> WebookSetEntityResponse:
        """
        Добавление нового слушателя webhook

        Args:
            webhook: объект WebhookRequest с параметрами запроса

        Returns:
            WebookSetEntityResponse: объект с информацией о добавленном
            слушателе webhook

        Raises:
            ValueError: если webhook не является объектом WebhookRequest
        """
        if not isinstance(webhook, WebhookRequest):
            raise ValueError("webhook must be a WebhookRequest")
        response = self._post("webhooks", json=webhook)
        return WebookSetEntityResponse.model_validate(response)



from __future__ import annotations

from typing import Literal
from uuid import UUID

from pydantic import BaseModel, Field


class CityFilter(BaseModel):
    """Фильтр для городов"""

    name: str = Field(..., description="Наименование населенного пункта СДЭК")
    country_code: str | None = Field(
        None, description="Код страны в формате ISO_3166-1_alpha-2"
    )


class CityListFilter(BaseModel):
    """Фильтр для списка городов"""

    country_codes: str | None = Field(
        None, description="Массив кодов стран в формате ISO_3166-1_alpha-2"
    )
    region_code: int | None = Field(None, description="Код региона (справочник СДЭК)")
    kladr_region_code: str | None = Field(None, description="Код КЛАДР региона")
    kladr_code: str | None = Field(None, description="Код КЛАДР населенного пункта")
    fias_guid: UUID | None = Field(
        None, description="Уникальный идентификатор ФИАС населенного пункта"
    )
    postal_code: str | None = Field(None, description="Почтовый индекс")
    code: int | None = Field(None, description="Код населенного пункта СДЭК")
    city: str | None = Field(
        None,
        description="Название населенного пункта. Должно соответствовать полностью",
    )
    payment_limit: float | None = Field(
        None, description="Ограничение на сумму наложенного платежа"
    )
    size: int | None = Field(1000, description="Ограничение выборки результата")
    page: int | None = Field(
        0, description="Номер страницы выборки результата. Нумерация страниц с 0"
    )
    lang: str | None = Field(None, description="Язык локализации ответа")


class RegionFilter(BaseModel):
    """Фильтр для регионов"""

    country_codes: str | None = Field(
        None, description="Список кодов стран в формате ISO_3166-1_alpha-2"
    )
    size: int = Field(1000, description="Ограничение выборки результата")
    page: int = Field(0, description="Номер страницы выборки результата")
    lang: Literal["rus", "zho"] = Field("rus", description="Локализация")


class ZipFilter(BaseModel):
    """Фильтр для почтовых индексов"""

    city_code: int = Field(
        ..., description="Код города, которому принадлежат почтовые индексы"
    )


class CoordinatesFilter(BaseModel):
    """Фильтр для координат"""

    latitude: float = Field(..., description="Широта")
    longitude: float = Field(..., description="Долгота")



from .enums import WebhookType
from .requests import WebhookRequest
from .responses import (
    WebhookDeleteEntityResponse,
    WebhookResponse,
    WebhookUUIDEntityResponse,
    WebookSetEntityResponse,
)
from .webhook import WebhookApp

__all__ = [
    "WebhookApp",
    "WebhookRequest",
    "WebhookResponse",
    "WebhookUUIDEntityResponse",
    "WebhookDeleteEntityResponse",
    "WebookSetEntityResponse",
    "WebhookType",
]



from __future__ import annotations

from pydantic import BaseModel, Field

from .money import Money
from .seller import Seller


class Item(BaseModel):
    name: str = Field(..., description="Название товара")
    ware_key: str = Field(..., description="Артикул товара")
    marking: str | None = Field(None, description="Маркировка товара")
    payment: Money = Field(..., description="Сумма оплаты с НДС")
    weight: int = Field(..., description="Вес товара в граммах")
    weight_gross: int | None = Field(None, description="Вес брутто")
    amount: int = Field(..., description="Количество товара")
    name_i18n: str | None = Field(None, description="Название товара на другом языке")
    brand: str | None = Field(None, description="Бренд на иностранном языке")
    country_code: str | None = Field(
        None, description="Код страны производителя товара в формате ISO_3166-1_alpha-2"
    )
    material: int | None = Field(None, description="Материал товара")
    wifi_gsm: bool | None = Field(None, description="Признак наличия WiFi и GSM модема")
    url: str | None = Field(None, description="URL товара")
    seller: Seller | None = Field(None, description="Реквизиты истинного продавца")
    cost: float = Field(..., description="Стоимость товара")
    feacn_code: str | None = Field(None, description="Код ТН ВЭД")
    jewel_uin: str | None = Field(None, description="УИН ювелирного изделия")
    used_goods: bool | None = Field(
        None, description="Признак товара б/у, применим для c2c заказов"
    )



from ..app import App
from .filters import CheckFilter
from .responses import CheckResponse


class CheckApp(App):
    """Класс для работы с чеками"""

    def get(self, filter_params: CheckFilter) -> CheckResponse:
        """
        Метод используется для получения информации
        о чеке по заказу или за выбранный день

        Args:
            filter_params (CheckFilter): фильтр для получения
                информации о чеке по заказу или за выбранный день

        Returns:
            CheckResponse: объект с информацией о чеке по заказу или за выбранный день

        Raises:
            ValueError: если filter_params не является объектом CheckFilter
        """
        if not isinstance(filter_params, CheckFilter):
            raise ValueError("filter_params must be a CheckFilter")
        response = self._get("check", params=filter_params)
        return CheckResponse.model_validate(response)



from __future__ import annotations

from datetime import date as Date
from datetime import datetime as DateTime
from uuid import UUID

from pydantic import BaseModel, Field, field_serializer

from ..models.error import Error
from ..models.warning import WarningModel


class PaymentOrder(BaseModel):
    order_uuid: UUID = Field(
        ...,
        description="Идентификатор заказа в ИС СДЭК. "
        "Обязателен, если не указан номер заказа",
    )
    cdek_number: int = Field(
        ...,
        description="Номер заказа СДЭК. Обязателен, "
        "если не указан идентификатор заказа",
    )
    number: str = Field(..., description="Номер заказа в ИС Клиента")


class PaymentInfoResponse(BaseModel):
    orders: list[PaymentOrder] | None = Field(None, description="Список заказов")
    errors: list[Error] | None = Field(None, description="Список ошибок")
    warnings: list[WarningModel] | None = Field(
        None, description="Список предупреждений"
    )


class RegistryOrder(BaseModel):
    cdek_number: str = Field(
        ...,
        description="Номер заказа СДЭК. Обязателен, "
        "если не указан идентификатор заказа",
    )
    transfer_sum: float = Field(
        ..., description="Сумма к начислению (в валюте взаиморасчетов)"
    )
    payment_sum: float = Field(
        ...,
        description="Сумма наложенного платежа, которую взяли с получателя"
        " (в валюте взаиморасчетов)",
    )
    total_sum_without_agent: float = Field(
        ...,
        description="Итоговая стоимость заказа без учета агентского вознаграждения"
        " (в валюте взаиморасчетов)",
    )
    agent_commission_sum: float = Field(
        ...,
        description="Агентское вознаграждение по переводу наложенного платежа "
        " (в валюте взаиморасчетов)",
    )


class Registry(BaseModel):
    registry_number: int = Field(..., description="Номер реестра наложенного платежа")
    payment_date: Date | None = Field(
        None, description="Фактическая дата оплаты реестра наложенного платежа"
    )
    sum: float = Field(..., description="Сумма по реестру (в валюте взаиморасчетов)")
    payment_order_number: str | None = Field(
        None,
        description=(
            "Номер платежного поручения, в рамках которого был осуществлен платеж. "
            "Если атрибут отсутствует или пустой, свяжитесь со своим менеджером"
        ),
    )
    orders: list[RegistryOrder] = Field(..., description="Список заказов реестра")
    date_created: DateTime | None = Field(
        None, description="Дата создания реестра наложенного платежа"
    )

    @field_serializer("date_created")
    def serialize_date_created(self, date_created: DateTime) -> str:
        """Вернуть дату создания реестра в формате ISO 8601"""
        return date_created.strftime("%Y-%m-%dT%H:%M:%S")

    @field_serializer("payment_date")
    def serialize_payment_date(self, payment_date: Date) -> str:
        """Представить дату оплаты реестра в формате YYYY-MM-DD"""
        return payment_date.strftime("%Y-%m-%d")


class PaymentResponse(BaseModel):
    registries: list[Registry] | None = Field(None, description="Список реестров")
    errors: list[Error] | None = Field(None, description="Список ошибок")
    warnings: list[WarningModel] | None = Field(
        None, description="Список предупреждений"
    )
